! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

! Module: srcmatch_def
!
! Authors:  Jacco Vink - AIP
!          
! Description: Definitions to be used in eident_mod & eident_io
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module srcmatch_def

  use param  
  use dal

  implicit none

  integer(kind=int32), parameter :: max_nsets = 60   !!! 9
  integer(kind=int32), parameter :: null_entry= 0
  integer(kind=int32), parameter :: instr_all = 0
  integer(kind=int32), parameter :: instr_pn = 1, instr_m1 = 2,instr_m2 = 3  
  integer(kind=int32), parameter :: summary_band = 0, xid_band=9
  real(kind=double), parameter :: pi = 3.1415926535897932385d0
  real(kind=double), parameter :: rad2deg = 180.0d0/pi
  real(kind=double), parameter :: deg2rad = pi/180.0d0
  real(kind=double), parameter :: rad2arcsec = 3600.0d0*180.0d0/pi
  real(kind=double), parameter :: arcsec2rad = pi/(3600.0d0*180.0d0)
  real(kind=double), parameter :: deg2arcsec = 3600.0d0
  real(kind=double), parameter :: sterrad2arcsec = 4.254517029d10
  real(kind=double), parameter :: ra_gal_pole = 192.25
  real(kind=double), parameter :: dec_gal_pole = 27.4
!  real(kind=double), parameter :: ra_gal_pole =  192.8594
!  real(kind=double), parameter :: ra_gal_pole =  192.864
!  real(kind=double), parameter :: ra_gal_pole = 192.859481
!  real(kind=double), parameter :: dec_gal_pole = 27.128251
  real(kind=double), parameter :: gal_theta=123.0d0
  real(kind=double) :: expos_pn,expos_m1,expos_m2
  character(40) :: subm_pn,subm_m1,subm_m2
  character(12) :: filt_pn,filt_m1,filt_m2
  logical :: xidflag,allepicflag, skycoord     !!! IT 2022: added skycoord

  !
  ! coordinate type definition, corresponds to XMM-SOC-ICD-0006-SSC,(p.43,v1.1)
  ! x,y, etc. corresponds to x_ima, y_ima etc.
  !
  type :: Source_type
     integer(kind=int32) :: id_src, id_inst, id_band
     integer(kind=int32) :: src_num, dummy_id
     integer(kind=int32),dimension(max_nsets) :: ep_ml_id_src,  pn_ml_id_src
     integer(kind=int32),dimension(max_nsets) :: m1_ml_id_src, m2_ml_id_src
     integer(kind=int32),dimension(max_nsets) :: ep_box_id_src, pn_box_id_src
     integer(kind=int32),dimension(max_nsets) :: m1_box_id_src, m2_box_id_src
     real(kind=double)   :: ra, dec, ra_corr, dec_corr
     real(kind=double)   :: lii, bii, lii_corr,bii_corr
     real(kind=single)  :: radec_err
     !!! IT 2015/07: copy image coordinates (SCR-7301)
     real(kind=single) :: x_ima, x_ima_err, y_ima, y_ima_err

     real(kind=single)  :: pn_tot, pn_tot_err
     real(kind=single)  :: m1_tot, m1_tot_err, m2_tot, m2_tot_err
     real(kind=single)  :: pn_scts,pn_scts_err,m1_scts, m1_scts_err, m2_scts, m2_scts_err
     real(kind=single)  :: ep_tot, ep_tot_err
     real(kind=single)  :: m1_1, m1_2, m1_3, m1_4, m1_5, m1_6
     real(kind=single)  :: m1_1_err, m1_2_err, m1_3_err, m1_4_err, m1_5_err, m1_6_err
     real(kind=single)  :: m2_1, m2_2, m2_3, m2_4, m2_5, m2_6
     real(kind=single)  :: m2_1_err, m2_2_err, m2_3_err, m2_4_err, m2_5_err, m2_6_err
     real(kind=single)  :: pn_1, pn_2, pn_3, pn_4, pn_5, pn_6
     real(kind=single)  :: pn_1_err, pn_2_err, pn_3_err, pn_4_err, pn_5_err, pn_6_err
     real(kind=single)  :: m1_1_exp, m1_2_exp, m1_3_exp, m1_4_exp, m1_5_exp, m1_6_exp
     real(kind=single)  :: m2_1_exp, m2_2_exp, m2_3_exp, m2_4_exp, m2_5_exp, m2_6_exp
     real(kind=single)  :: pn_1_exp, pn_2_exp, pn_3_exp, pn_4_exp, pn_5_exp, pn_6_exp

     real(kind=single)  :: m1_1_ml, m1_2_ml, m1_3_ml, m1_4_ml, m1_5_ml, m1_6_ml
     real(kind=single)  :: m2_1_ml, m2_2_ml, m2_3_ml, m2_4_ml, m2_5_ml, m2_6_ml
     real(kind=single)  :: pn_1_ml, pn_2_ml, pn_3_ml, pn_4_ml, pn_5_ml, pn_6_ml

     real(kind=single)  :: m1_1_vig, m1_2_vig, m1_3_vig, m1_4_vig, m1_5_vig, m1_6_vig
     real(kind=single)  :: m2_1_vig, m2_2_vig, m2_3_vig, m2_4_vig, m2_5_vig, m2_6_vig
     real(kind=single)  :: pn_1_vig, pn_2_vig, pn_3_vig, pn_4_vig, pn_5_vig, pn_6_vig
     real(kind=single)  :: pn_ontime, m1_ontime, m2_ontime
     real(kind=single)  :: pn_maskfrac, m1_maskfrac, m2_maskfrac



     real(kind=single)  ::  pn_1_flux, pn_2_flux, pn_3_flux, pn_4_flux, pn_5_flux, pn_6_flux  
     real(kind=single)  ::  pn_tot_flux,  pn_xid_flux
     real(kind=single)  ::  m1_1_flux, m1_2_flux, m1_3_flux, m1_4_flux, m1_5_flux, m1_6_flux  
     real(kind=single)  ::  m1_tot_flux,  m1_xid_flux
     real(kind=single)  ::  m2_1_flux, m2_2_flux, m2_3_flux, m2_4_flux, m2_5_flux, m2_6_flux  
     real(kind=single)  ::  m2_tot_flux,  m2_xid_flux

     real(kind=single)  ::  err_pn_1_flux, err_pn_2_flux, err_pn_3_flux, err_pn_4_flux, err_pn_5_flux, err_pn_6_flux  
     real(kind=single)  ::  err_pn_tot_flux,  err_pn_xid_flux
     real(kind=single)  ::  err_m1_1_flux, err_m1_2_flux, err_m1_3_flux, err_m1_4_flux, err_m1_5_flux, err_m1_6_flux  
     real(kind=single)  ::  err_m1_tot_flux,  err_m1_xid_flux
     real(kind=single)  ::  err_m2_1_flux, err_m2_2_flux, err_m2_3_flux, err_m2_4_flux, err_m2_5_flux, err_m2_6_flux  
     real(kind=single)  ::  err_m2_tot_flux,  err_m2_xid_flux

     real(kind=single)  ::  ep_1_flux, ep_2_flux, ep_3_flux, ep_4_flux, ep_5_flux 
     real(kind=single)  ::  ep_tot_flux,  ep_xid_flux
     real(kind=single)  ::  err_ep_1_flux, err_ep_2_flux, err_ep_3_flux, err_ep_4_flux, err_ep_5_flux  
     real(kind=single)  ::  err_ep_tot_flux,  err_ep_xid_flux
    
     real(kind=single)  :: pn_xid, pn_xid_err
     real(kind=single)  :: m1_xid, m1_xid_err, m2_xid, m2_xid_err
     real(kind=single)  :: ep_xid, ep_xid_err
     real(kind=single),dimension(max_nsets) :: pn_det_ml, m1_det_ml, m2_det_ml
     real(kind=single),dimension(max_nsets) :: pn_xid_ml, m1_xid_ml, m2_xid_ml
     real(kind=single),dimension(max_nsets) :: ep_det_ml,ep_xid_ml
     real(kind=single)  :: pn_extent, pn_ext_err
     real(kind=single)  :: m1_extent, m1_ext_err
     real(kind=single)  :: m2_extent, m2_ext_err
     real(kind=single)  :: ep_extent, ep_ext_err
     real(kind=single),dimension(max_nsets) :: pn_ext_ml, m1_ext_ml, m2_ext_ml
     real(kind=single),dimension(max_nsets) :: ep_ext_ml
     real(kind=single)  :: pn_offax, m1_offax, m2_offax
     real(kind=single)  :: pn_hr1, pn_hr1_err, pn_hr2, pn_hr2_err
     real(kind=single)  :: pn_hr3, pn_hr3_err, pn_hr4, pn_hr4_err
     real(kind=single)  :: m1_hr1, m1_hr1_err, m1_hr2, m1_hr2_err
     real(kind=single)  :: m1_hr3, m1_hr3_err, m1_hr4, m1_hr4_err 
     real(kind=single)  :: m2_hr1, m2_hr1_err, m2_hr2, m2_hr2_err
     real(kind=single)  :: m2_hr3, m2_hr3_err, m2_hr4, m2_hr4_err
     real(kind=single)  :: ep_hr1, ep_hr1_err, ep_hr2, ep_hr2_err
     real(kind=single)  :: ep_hr3, ep_hr3_err, ep_hr4, ep_hr4_err
     real(kind=single)  :: pn_pileup, m1_pileup, m2_pileup       !!! IT 2019/02
     real(kind=single)  :: var_stat, p_var
     integer(kind=int32) :: n_bins, var_exp_no
     integer(kind=int32),dimension(5) :: om_id
     real(kind=single),dimension(5)   :: om_id_dist
     integer(kind=int32):: n_opt_co
     character(10)      :: omflag
     logical            :: spectra, tseries
     character(12)      :: m1_val_flag,  m2_val_flag,  pn_val_flag
     character(12)      :: m1_ver_flag,  m2_ver_flag,  pn_ver_flag
     character(12)      :: ep_flag
     character(40)      :: m1_ver_comm,  m2_ver_comm,  pn_ver_comm
     integer(kind=int32):: cross_ref
     real(kind=single)  :: dist
  end type Source_type

  type :: Header_type
     character(200),dimension(max_nsets) :: inputsets, crossidsets
     character(72)       :: telescop, instrume, filter, detnam, datamode
     character(72)       :: obs_mode,obs_id, exp_id, content, origin, object
     character(72)       :: filt_pn, filt_m1, filt_m2
     character(72)       :: subm_pn, subm_m1, subm_m2
     character(72)       :: date_obs, date_end, att_src
     character(72)       :: radecsys, observer
     logical             :: omcross,single_instr,xidlist
     integer(kind=int32),dimension(max_nsets) :: id_inst
     integer(kind=int32) :: nsets, ncrossid
     real(kind=single)   :: exposure, expos_pn, expos_m1,expos_m2, equinox
     real(kind=single)   :: imbinsize      !!! IT 2022: for extentunit
     real(kind=double)   :: ra_pnt, dec_pnt, pa_pnt
     real(kind=double)   :: max_err, syst_err, om_dist
     real(kind=single)   :: raoffset, deoffset, raofferr, deofferr
     real(kind=single)   :: rot_corr, rot_err, lik_hood, nmatches
     real(kind=single), dimension(3,9) :: eblo, ebhi
  end type Header_type

  type :: Sheader_type
     integer(kind=int32)  :: dummy
  end type Sheader_type



end module srcmatch_def



