! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!+
! Module: srcmatch_io
!
! Authors:  Jacco Vink - AIP
!           Dieter Meinert - AIP (D.M.)
!           Georg Lamer (AIP)
!          
! Description: I/O routines to be used in srcmatch_mod
! ! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module srcmatch_io

  use param  
  use dal
  use srcmatch_def


  implicit none

  private

  public read_set, read_Header, read_om_set, write_summary_set, &
       write_summary_header, write_primary_header,write_html_list, write_html_xid, init_list


contains


  !--------------------------------------------------------------------
  function read_set( filename, n_sources, has_ra_corr, Header ) &
       result( list_ptr)
  !--------------------------------------------------------------------

    type(Source_type), dimension(:), pointer :: list_ptr
    character(*), intent(in) :: filename
    integer(kind=int32) , intent (out) :: n_sources
    logical , intent (out)             :: has_ra_corr
    type(Header_type) , intent(out) :: Header

    logical :: single_instr = .TRUE.
    logical :: single_band  = .TRUE.
    logical :: no_instsumm = .TRUE.
    logical :: xidlist
    integer(kind=int32) :: i, j, nrows, colnr, oldinstr, ctype
    integer(kind=int32),dimension(:),pointer :: src_num,id_src_ml,id_src_box
    integer(kind=int32),dimension(:),pointer :: instrument, id_band
    real(kind=double),dimension(:),pointer :: instrtmp !!! IT 2016: may be real
    real(kind=double),dimension(:),pointer :: ra, dec, ra_corr, dec_corr
    real(kind=single),dimension(:),pointer :: radec_err, offax
    real(kind=single),dimension(:),pointer :: rate, rate_err, det_ml,scts, scts_err
    !!! IT 2015/07: copy image coordinates (SCR-7301)
    real(kind=single),dimension(:),pointer :: x_ima, x_ima_err, y_ima, y_ima_err
    real(kind=single),dimension(:),pointer :: ext, ext_err, ext_ml
    real(kind=single),dimension(:),pointer :: exp_map, flux, flux_err, vignetting, ontime, maskfrac
    real(kind=single),dimension(:),pointer :: hr1, hr1_err, hr2, hr2_err
    real(kind=single),dimension(:),pointer :: hr3, hr3_err,hr4, hr4_err
    !!! IT 2019/02: copy pile-up levels
    real(kind=single),dimension(:),pointer :: pileup
!    real(kind=single),dimension(:), allocatable :: realdummy
    character(12),dimension(:),pointer     :: ver_flag, val_flag
    character(40),dimension(:),pointer     :: ver_comm


    type(DataSetT) :: set
    type(TableT)  :: source_tab
    type(ColumnT),dimension(300) :: col

    set = dataSet(filename,READ)
    call read_Header(set, Header )

    source_tab = table( set,'SRCLIST')

    ! get number of rows in this table
    nrows = numberOfRows(source_tab)
    !3.7.3 if( nrows <= 0 ) then
    !        call release(source_tab) ! probably superfluous
    !        call release(set)        ! probably superfluous
    !        call error('emptyInputSet','EMLDETECT source list is empty')
    !3.7.3 end if

    allocate(ver_flag(0:nrows-1))
    allocate(val_flag(0:nrows-1))
    allocate(ver_comm(0:nrows-1))

    colnr=0

    colnr = colnr + 1
    if (hasColumn(source_tab,"ML_ID_SRC")) then      !!! IT 2016/02: eboxdetect
       col(colnr) = column(source_tab, "ML_ID_SRC",READ)
       src_num => int32Data( col(colnr) )
       colnr = colnr + 1
    else
       col(colnr) = column(source_tab, "BOX_ID_SRC",READ)
      src_num => int32Data( col(colnr) )
       colnr = colnr + 1
    endif
    if (hasColumn(source_tab,"ML_ID_SRC")) then      !!! IT 2016/02: eboxdetect
       col(colnr) = column(source_tab, "ML_ID_SRC",READ)
       id_src_ml => int32Data( col(colnr) )
       colnr = colnr + 1
    else
       allocate(id_src_ml(0:nrows-1))
    endif
    col(colnr) = column(source_tab, "BOX_ID_SRC",READ)
    id_src_box => int32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab, "ID_INST",READ)  !!! IT 2016: may be real
    ctype = columnDataType(col(colnr))
    if (ctype == Real64) then
       instrtmp => real64Data( col(colnr) )
       allocate(instrument(0:nrows-1))
       instrument = nint(instrtmp)
    else
       instrument => int32Data( col(colnr) )
    endif
    colnr = colnr + 1
    col(colnr) = column(source_tab, "ID_BAND",READ)
    id_band => int32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab, "SCTS",READ)
    scts => real32Data( col(colnr) )
    col(colnr) = column(source_tab, "SCTS_ERR",READ)
    scts_err => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"X_IMA", READ)
    x_ima => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"X_IMA_ERR", READ)
    x_ima_err => real32Data( col(colnr))
    colnr = colnr + 1
    col(colnr) = column(source_tab,"Y_IMA", READ)
    y_ima => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"Y_IMA_ERR", READ)
    y_ima_err => real32Data( col(colnr))
    colnr = colnr + 1
    col(colnr) = column(source_tab, "RA",READ)
    ra => real64Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"DEC", READ)
    dec => real64Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"RADEC_ERR", READ)
    radec_err => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"FLUX", READ)
    flux => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"FLUX_ERR", READ)
    flux_err => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"RATE", READ)
    rate => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"RATE_ERR", READ)
    rate_err => real32Data( col(colnr))
    if (hasColumn(source_tab,"DET_ML")) then           !!! IT 2016/02: eboxdetect
       colnr = colnr + 1
       col(colnr) = column(source_tab,"DET_ML", READ)
       det_ml =>real32Data( col(colnr) )
    else
       if (hasColumn(source_tab,"LIKE")) then
          colnr = colnr + 1
          col(colnr) = column(source_tab,"LIKE", READ)
          det_ml =>real32Data( col(colnr) )
       else
          allocate(det_ml(0:nrows-1)) 
       endif
    endif
    if (hasColumn(source_tab,"OFFAX")) then
      colnr = colnr + 1
      col(colnr) = column(source_tab,"OFFAX", READ)
      offax => real32Data( col(colnr) )
    else
       allocate(offax(0:nrows-1)) 
    endif
    colnr = colnr + 1
    col(colnr) = column(source_tab,"HR1", READ)
    hr1 => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"HR1_ERR", READ)
    hr1_err => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"HR2", READ)
    hr2 => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"HR2_ERR", READ)
    hr2_err => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"HR3", READ)
    hr3 => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"HR3_ERR", READ)
    hr3_err => real32Data( col(colnr) )
    if (hasColumn(source_tab,"HR4")) then
      colnr = colnr + 1
      col(colnr) = column(source_tab,"HR4", READ)
      hr4 => real32Data( col(colnr) )
    else
      allocate(hr4(0:nrows-1))
    endif
    if (hasColumn(source_tab,"HR4_ERR")) then
      colnr = colnr + 1
      col(colnr) = column(source_tab,"HR4_ERR", READ)
      hr4_err => real32Data( col(colnr) )
      colnr = colnr + 1
     else
       allocate(hr4_err(0:nrows-1))
     endif
    if (hasColumn(source_tab,"EXT")) then        !!! IT 2016/02: eboxdetect
       ! warning: not checking for the validity of the binsize keywords
       col(colnr) = column(source_tab,"EXT", READ)
       ext =>real32Data( col(colnr) )
       colnr = colnr + 1
       col(colnr) = column(source_tab,"EXT_ERR", READ)
       ext_err =>real32Data( col(colnr) )
       colnr = colnr + 1
       col(colnr) = column(source_tab,"EXT_ML", READ)
       ext_ml =>real32Data( col(colnr) )
       colnr = colnr + 1
       col(colnr) = column(source_tab,"EXP_MAP", READ)
       exp_map =>real32Data( col(colnr) )
       if (skycoord) then
          ext     = ext     * Header%imbinsize
          ext_err = ext_err * Header%imbinsize
       endif
    else
       allocate(ext(0:nrows-1))
       allocate(ext_err(0:nrows-1))
       allocate(ext_ml(0:nrows-1))
       allocate(exp_map(0:nrows-1))  
    endif
    if (hasColumn(source_tab,"VIGNETTING")) then
      colnr = colnr + 1
      col(colnr) = column(source_tab,"VIGNETTING", READ)
      vignetting =>real32Data( col(colnr) )
    else
      allocate(vignetting(0:nrows-1))
    endif
    if (hasColumn(source_tab,"ONTIME")) then
      colnr = colnr + 1
      col(colnr) = column(source_tab,"ONTIME", READ)
      ontime =>real32Data( col(colnr) )
     else
      allocate(ontime(0:nrows-1)) 
    endif
    if (hasColumn(source_tab,"MASKFRAC")) then
      colnr = colnr + 1
      col(colnr) = column(source_tab,"MASKFRAC", READ)
      maskfrac =>real32Data( col(colnr) )
     else
      allocate(maskfrac(0:nrows-1)) 
    endif    
    if (hasColumn(source_tab,"PILEUP")) then     !!! IT 2019/02
      colnr = colnr + 1
      col(colnr) = column(source_tab,"PILEUP", READ)
      pileup => real32Data( col(colnr) )
    else
      allocate(pileup(0:nrows-1))
    endif
    if (hasColumn(source_tab,"VAL_FLAG")) then
      colnr = colnr + 1
      col(colnr) = column(source_tab,"VAL_FLAG", READ)   
      do i=0,nrows-1
        val_flag(i)=stringCell(col(colnr),i)
      end do
    else if (hasColumn(source_tab,"FLAG")) then
      colnr = colnr + 1
      col(colnr) = column(source_tab,"FLAG", READ)   
      do i=0,nrows-1
        val_flag(i)=stringCell(col(colnr),i)
      end do  
    endif
    if (hasColumn(source_tab,"VER_FLAG")) then
      colnr = colnr + 1
      col(colnr) = column(source_tab,"VER_FLAG", READ)   
      do i=0,nrows-1
        ver_flag(i)=stringCell(col(colnr),i)
      end do
!    else
!      if (hasColumn(source_tab,"FLAG")) then
!        do i=0,nrows-1
!          ver_flag(i)=stringCell(col(colnr),i)
!        end do
!      endif
    endif
    if (hasColumn(source_tab,"VER_COMM")) then
      colnr = colnr + 1
      col(colnr) = column(source_tab,"VER_COMM", READ)   
      do i=0,nrows-1
        ver_comm(i)=stringCell(col(colnr),i)
      end do
    endif


    has_ra_corr = .FALSE.
    if(  hasAttribute(source_tab,"LIK_HOOD") ) then !eposcorr has been applied
       Header%lik_hood = real32Attribute(source_tab,"LIK_HOOD")
       Header%raoffset = real32Attribute(source_tab,"RAOFFSET")
       Header%deoffset = real32Attribute(source_tab,"DEOFFSET")
       Header%raofferr = real32Attribute(source_tab,"RAOFFERR")
       Header%deofferr = real32Attribute(source_tab,"DEOFFERR")
       Header%rot_corr = real32Attribute(source_tab,"ROT_CORR")
       Header%rot_err  = real32Attribute(source_tab,"ROT_ERR")
       Header%nmatches = real32Attribute(source_tab,"NMATCHES")
       colnr = colnr + 1
       col(colnr) = column(source_tab, "RA_CORR",READ)
       ra_corr => real64Data( col(colnr) )
       colnr = colnr + 1
       col(colnr) = column(source_tab,"DEC_CORR", READ)
       dec_corr => real64Data( col(colnr) )
       has_ra_corr = .TRUE.
    end if

! Due to the complicated input list first a number of checks need to
! be done.
! The first one is to check whether there exist rows with 
! instrument(i-1) == instr_all. If not it is a single instrument source list.
!

 
! need to change this due to new emldetect format with ID_INST=0 row for single instr list (20/09/05):
    
!    single_instr = .TRUE.
!    Header%single_instr = .TRUE.
!    do i=1, nrows
!       if( instrument(i-1) == instr_all ) then
!          single_instr = .FALSE.
!          Header%single_instr = .FALSE.
!          exit
!       end if
!    end do

    !!! IT 2014/09: use only for old-style source lists without summary rows
    single_instr = .TRUE.
    oldinstr = instr_all
    do i=1, nrows
       !!! IT 2014/09: use only for old-style source lists without summary rows
       if (instrument(i-1) == instr_all) then
          no_instsumm = .FALSE.
          single_instr = .FALSE.
          exit
       end if
       if( (instrument(i-1) /= oldinstr) .and. (oldinstr /= instr_all) ) then
          single_instr = .FALSE.
          exit
       end if
       oldinstr = instrument(i-1)
    end do
    Header%single_instr = single_instr




!    check if this is an XID band source list:  

    xidlist = .false.
    do i=1, nrows
       if(id_band(i-1)  .eq. xid_band)  then 
         xidlist=.true.
         xidflag=.true.
       endif
       if( id_band(i-1) .ne. summary_band .and. id_band(i-1) .ne. xid_band) then
          xidlist = .false.
          Header%xidlist = .false.
          exit
       end if
    end do

    if (single_instr .and. nrows>0) then   ! 3.7.3
      Header%id_inst(1) = instrument(1)
    else
      Header%id_inst(1) = 0
    end if 

! idem but now for the energy band, check whether summary_band info exists
    single_band = .TRUE.
    do i=1, nrows
       if( id_band(i-1) == summary_band ) then
          single_band = .FALSE.          
          exit
       end if
    end do

    ! allocate position list
    allocate(list_ptr(nrows))
    call init_list(list_ptr, nrows)
    n_sources =0 

    do i=1, nrows
       ! remove single_instr (20/9/05)
       if( ((instrument(i-1) == instr_all) .OR. no_instsumm) .AND. &
            (id_band(i-1) == summary_band .OR. single_band) ) then
          n_sources = n_sources + 1
          list_ptr(i)%dummy_id = n_sources
          list_ptr(i)%src_num  = src_num(i-1)
       else
          list_ptr(i)%dummy_id = 0
       end if
    end do

!    print "(A I4)", 'nsrcs= ',   n_sources 

! associate all row entries with a source number:
!
    do i=1, nrows
       if(list_ptr(i)%dummy_id == 0 ) then
          do j=1,nrows
             if( src_num(i-1) == list_ptr(j)%src_num ) then
                list_ptr(i)%dummy_id = list_ptr(j)%dummy_id
             end if
          end do
       end if
    end do

    ! Copy the fits data into the structure
    !
    n_sources = 0
    do i=1, nrows
       ! the -1 is the diff. between F90 and C(++)
       j = list_ptr(i)%dummy_id
       if( (instrument(i-1) == instr_all .OR. no_instsumm ) .AND. &
            (id_band(i-1) == summary_band .OR. single_band) ) then
          n_sources = n_sources + 1
          
          list_ptr(j)%cross_ref = 0 ! initialize
          list_ptr(j)%src_num = src_num(i-1) 
          list_ptr(j)%ra = ra(i-1) 
          list_ptr(j)%dec = dec(i-1)
          !catch NaNs, necessary for XID band lists (21/01/05):
          if (.not. (radec_err(i-1) .ge. 0.0)) then
            list_ptr(j)%radec_err = 0.0
          else
            list_ptr(j)%radec_err = radec_err(i-1)
          endif
          if( has_ra_corr ) then 
             list_ptr(j)%ra_corr = ra_corr(i-1) 
             list_ptr(j)%dec_corr = dec_corr(i-1)
          end if
          list_ptr(j)%x_ima     = x_ima(i-1) 
          list_ptr(j)%x_ima_err = x_ima_err(i-1)             
          list_ptr(j)%y_ima     = y_ima(i-1) 
          list_ptr(j)%y_ima_err = y_ima_err(i-1)             
       end if

!      if( id_band(i-1) == summary_band) then
!      included single band case here as well (24/02/03, GL)



     
       if( id_band(i-1) == summary_band .or. single_band) then
          if( instrument(i-1) == instr_all) then
             list_ptr(j)%ep_ml_id_src(1) =  id_src_ml(i-1)
             list_ptr(j)%ep_box_id_src(1) =  id_src_box(i-1)
             if (.not. xidlist) then
               list_ptr(j)%ep_tot     = rate(i-1) 
               list_ptr(j)%ep_tot_err = rate_err(i-1) 
             endif
             list_ptr(j)%ep_det_ml(1)  = det_ml(i-1)
             list_ptr(j)%ep_extent  = ext(i-1)
             list_ptr(j)%ep_ext_err = ext_err(i-1)
             list_ptr(j)%ep_ext_ml(1) = ext_ml(i-1)
             list_ptr(j)%ep_hr1     = hr1(i-1)
             list_ptr(j)%ep_hr1_err = hr1_err(i-1)
             list_ptr(j)%ep_hr2     = hr2(i-1)
             list_ptr(j)%ep_hr2_err = hr2_err(i-1)
             list_ptr(j)%ep_hr3     = hr3(i-1)
             list_ptr(j)%ep_hr3_err = hr3_err(i-1)
             list_ptr(j)%ep_hr4     = hr4(i-1)
             list_ptr(j)%ep_hr4_err = hr4_err(i-1)
             if (.not. xidlist) then
               list_ptr(j)%ep_flag = val_flag(i-1) 
             endif
          end if
          if( (instrument(i-1) == instr_all .and. xidlist) .or. (single_band .and. xidlist) ) then
            list_ptr(j)%ep_xid_ml(1)  = det_ml(i-1)
          endif
     
          if( instrument(i-1) == instr_pn) then
             list_ptr(j)%pn_ml_id_src(1) =  id_src_ml(i-1)
             list_ptr(j)%pn_box_id_src(1) =  id_src_box(i-1)
             list_ptr(j)%pn_tot_flux     = flux(i-1) 
             list_ptr(j)%err_pn_tot_flux = flux_err(i-1)
             list_ptr(j)%pn_tot     = rate(i-1) 
             list_ptr(j)%pn_tot_err = rate_err(i-1)
             list_ptr(j)%pn_scts     = scts(i-1) 
             list_ptr(j)%pn_scts_err = scts_err(i-1)             
             list_ptr(j)%pn_det_ml(1)  = det_ml(i-1)
             list_ptr(j)%pn_extent  = ext(i-1)
             list_ptr(j)%pn_ext_err = ext_err(i-1)
             list_ptr(j)%pn_ext_ml(1)  = ext_ml(i-1)
             list_ptr(j)%pn_offax     = offax(i-1)             
             list_ptr(j)%pn_hr1     = hr1(i-1)
             list_ptr(j)%pn_hr1_err = hr1_err(i-1)
             list_ptr(j)%pn_hr2     = hr2(i-1)
             list_ptr(j)%pn_hr2_err = hr2_err(i-1)
             list_ptr(j)%pn_hr3     = hr3(i-1)
             list_ptr(j)%pn_hr3_err = hr3_err(i-1)
             list_ptr(j)%pn_hr4     = hr4(i-1)
             list_ptr(j)%pn_hr4_err = hr4_err(i-1)
             list_ptr(j)%pn_pileup  = pileup(i-1)
             list_ptr(j)%pn_ontime  = ontime(i-1)
             list_ptr(j)%pn_maskfrac  = maskfrac(i-1)
             list_ptr(j)%pn_val_flag = val_flag(i-1)
             list_ptr(j)%pn_ver_flag = ver_flag(i-1)
             list_ptr(j)%pn_ver_comm = ver_comm(i-1)
          end if
          if( instrument(i-1) == instr_m1) then
             list_ptr(j)%m1_ml_id_src(1) =  id_src_ml(i-1)
             list_ptr(j)%m1_box_id_src(1) =  id_src_box(i-1)
             list_ptr(j)%m1_tot_flux     = flux(i-1) 
             list_ptr(j)%err_m1_tot_flux = flux_err(i-1)
             list_ptr(j)%m1_tot     = rate(i-1) 
             list_ptr(j)%m1_tot_err = rate_err(i-1) 
             list_ptr(j)%m1_scts     = scts(i-1) 
             list_ptr(j)%m1_scts_err = scts_err(i-1)  
             list_ptr(j)%m1_det_ml(1) = det_ml(i-1)
             list_ptr(j)%m1_extent  = ext(i-1)
             list_ptr(j)%m1_ext_err = ext_err(i-1)
             list_ptr(j)%m1_ext_ml(1) = ext_ml(i-1)
             list_ptr(j)%m1_offax     = offax(i-1) 
             list_ptr(j)%m1_hr1     = hr1(i-1)
             list_ptr(j)%m1_hr1_err = hr1_err(i-1)
             list_ptr(j)%m1_hr2     = hr2(i-1)
             list_ptr(j)%m1_hr2_err = hr2_err(i-1)
             list_ptr(j)%m1_hr3     = hr3(i-1)
             list_ptr(j)%m1_hr3_err = hr3_err(i-1)
             list_ptr(j)%m1_hr4     = hr4(i-1)
             list_ptr(j)%m1_hr4_err = hr4_err(i-1)
             list_ptr(j)%m1_pileup  = pileup(i-1)
             list_ptr(j)%m1_ontime  = ontime(i-1)
             list_ptr(j)%m1_maskfrac  = maskfrac(i-1)
             list_ptr(j)%m1_val_flag = val_flag(i-1)
             list_ptr(j)%m1_ver_flag = ver_flag(i-1)
             list_ptr(j)%m1_ver_comm = ver_comm (i-1)
          end if
          if( instrument(i-1) == instr_m2) then
             list_ptr(j)%m2_ml_id_src(1) =  id_src_ml(i-1)
             list_ptr(j)%m2_box_id_src(1) =  id_src_box(i-1)
             list_ptr(j)%m2_tot_flux     = flux(i-1) 
             list_ptr(j)%err_m2_tot_flux = flux_err(i-1)
             list_ptr(j)%m2_tot     = rate(i-1) 
             list_ptr(j)%m2_tot_err = rate_err(i-1) 
             list_ptr(j)%m2_scts     = scts(i-1) 
             list_ptr(j)%m2_scts_err = scts_err(i-1) 
             list_ptr(j)%m2_det_ml(1)  = det_ml(i-1)
             list_ptr(j)%m2_extent  = ext(i-1)
             list_ptr(j)%m2_ext_err = ext_err(i-1)
             list_ptr(j)%m2_ext_ml(1)  = ext_ml(i-1)
             list_ptr(j)%m2_offax     = offax(i-1) 
             list_ptr(j)%m2_hr1     = hr1(i-1)
             list_ptr(j)%m2_hr1_err = hr1_err(i-1)
             list_ptr(j)%m2_hr2     = hr2(i-1)
             list_ptr(j)%m2_hr2_err = hr2_err(i-1)
             list_ptr(j)%m2_hr3     = hr3(i-1)
             list_ptr(j)%m2_hr3_err = hr3_err(i-1)
             list_ptr(j)%m2_hr4     = hr4(i-1)
             list_ptr(j)%m2_hr4_err = hr4_err(i-1)
             list_ptr(j)%m2_pileup  = pileup(i-1)
             list_ptr(j)%m2_ontime  = ontime(i-1)
             list_ptr(j)%m2_maskfrac  = maskfrac(i-1)
             list_ptr(j)%m2_val_flag = val_flag(i-1)
             list_ptr(j)%m2_ver_flag = ver_flag(i-1)
             list_ptr(j)%m2_ver_comm = ver_comm(i-1)
          end if
       end if


       if( id_band(i-1) == 1) then
          if( instrument(i-1) == instr_pn) then
             list_ptr(j)%pn_1_flux     = flux(i-1) 
             list_ptr(j)%err_pn_1_flux = flux_err(i-1)
             list_ptr(j)%pn_1     = rate(i-1) 
             list_ptr(j)%pn_1_err = rate_err(i-1)
             list_ptr(j)%pn_1_exp = exp_map(i-1)
             list_ptr(j)%pn_1_vig = vignetting(i-1)
             list_ptr(j)%pn_1_ml = det_ml(i-1)
          end if
          if( instrument(i-1) == instr_m1) then
             list_ptr(j)%m1_1_flux     = flux(i-1) 
             list_ptr(j)%err_m1_1_flux = flux_err(i-1)
             list_ptr(j)%m1_1     = rate(i-1) 
             list_ptr(j)%m1_1_err = rate_err(i-1) 
             list_ptr(j)%m1_1_exp = exp_map(i-1)
             list_ptr(j)%m1_1_vig = vignetting(i-1)
             list_ptr(j)%m1_1_ml = det_ml(i-1)
          end if
          if( instrument(i-1) == instr_m2) then
             list_ptr(j)%m2_1_flux     = flux(i-1) 
             list_ptr(j)%err_m2_1_flux = flux_err(i-1)
             list_ptr(j)%m2_1     = rate(i-1) 
             list_ptr(j)%m2_1_err = rate_err(i-1) 
             list_ptr(j)%m2_1_exp = exp_map(i-1) 
             list_ptr(j)%m2_1_vig = vignetting(i-1) 
             list_ptr(j)%m2_1_ml = det_ml(i-1)
          end if
       end if


       if( id_band(i-1) == 2) then
          if( instrument(i-1) == instr_pn) then
             list_ptr(j)%pn_2_flux     = flux(i-1) 
             list_ptr(j)%err_pn_2_flux = flux_err(i-1)
             list_ptr(j)%pn_2     = rate(i-1) 
             list_ptr(j)%pn_2_err = rate_err(i-1)
             list_ptr(j)%pn_2_exp = exp_map(i-1)
             list_ptr(j)%pn_2_vig = vignetting(i-1)
             list_ptr(j)%pn_2_ml = det_ml(i-1)
          end if
          if( instrument(i-1) == instr_m1) then
             list_ptr(j)%m1_2_flux     = flux(i-1) 
             list_ptr(j)%err_m1_2_flux = flux_err(i-1)
             list_ptr(j)%m1_2     = rate(i-1) 
             list_ptr(j)%m1_2_err = rate_err(i-1) 
             list_ptr(j)%m1_2_exp = exp_map(i-1)
             list_ptr(j)%m1_2_vig = vignetting(i-1)
             list_ptr(j)%m1_2_ml = det_ml(i-1)
          end if
          if( instrument(i-1) == instr_m2) then
             list_ptr(j)%m2_2_flux     = flux(i-1) 
             list_ptr(j)%err_m2_2_flux = flux_err(i-1)
             list_ptr(j)%m2_2     = rate(i-1) 
             list_ptr(j)%m2_2_err = rate_err(i-1) 
             list_ptr(j)%m2_2_exp = exp_map(i-1)
             list_ptr(j)%m2_2_vig = vignetting(i-1) 
             list_ptr(j)%m2_2_ml = det_ml(i-1)
         end if
       end if



       if( id_band(i-1) == 3) then
          if( instrument(i-1) == instr_pn) then
             list_ptr(j)%pn_3_flux     = flux(i-1) 
             list_ptr(j)%err_pn_3_flux = flux_err(i-1)
             list_ptr(j)%pn_3     = rate(i-1) 
             list_ptr(j)%pn_3_err = rate_err(i-1)
             list_ptr(j)%pn_3_exp = exp_map(i-1)
             list_ptr(j)%pn_3_vig = vignetting(i-1) 
             list_ptr(j)%pn_3_ml = det_ml(i-1)
          end if
          if( instrument(i-1) == instr_m1) then
             list_ptr(j)%m1_3_flux     = flux(i-1) 
             list_ptr(j)%err_m1_3_flux = flux_err(i-1)
             list_ptr(j)%m1_3     = rate(i-1) 
             list_ptr(j)%m1_3_err = rate_err(i-1) 
             list_ptr(j)%m1_3_exp = exp_map(i-1)
             list_ptr(j)%m1_3_vig = vignetting(i-1) 
             list_ptr(j)%m1_3_ml = det_ml(i-1)
          end if
          if( instrument(i-1) == instr_m2) then
             list_ptr(j)%m2_3_flux     = flux(i-1) 
             list_ptr(j)%err_m2_3_flux = flux_err(i-1)
             list_ptr(j)%m2_3     = rate(i-1) 
             list_ptr(j)%m2_3_err = rate_err(i-1) 
             list_ptr(j)%m2_3_exp = exp_map(i-1)
             list_ptr(j)%m2_3_vig = vignetting(i-1) 
             list_ptr(j)%m2_3_ml = det_ml(i-1)
          end if
       end if


       if( id_band(i-1) == 4) then
          if( instrument(i-1) == instr_pn) then
             list_ptr(j)%pn_4_flux     = flux(i-1) 
             list_ptr(j)%err_pn_4_flux = flux_err(i-1)
             list_ptr(j)%pn_4     = rate(i-1) 
             list_ptr(j)%pn_4_err = rate_err(i-1)
             list_ptr(j)%pn_4_exp = exp_map(i-1)
             list_ptr(j)%pn_4_vig = vignetting(i-1)
             list_ptr(j)%pn_4_ml = det_ml(i-1)
          end if
          if( instrument(i-1) == instr_m1) then
            list_ptr(j)%m1_4_flux     = flux(i-1) 
             list_ptr(j)%err_m1_4_flux = flux_err(i-1)
             list_ptr(j)%m1_4     = rate(i-1) 
             list_ptr(j)%m1_4_err = rate_err(i-1) 
             list_ptr(j)%m1_4_exp = exp_map(i-1)
             list_ptr(j)%m1_4_vig = vignetting(i-1) 
             list_ptr(j)%m1_4_ml = det_ml(i-1)
          end if
          if( instrument(i-1) == instr_m2) then
             list_ptr(j)%m2_4_flux     = flux(i-1) 
             list_ptr(j)%err_m2_4_flux = flux_err(i-1)
             list_ptr(j)%m2_4     = rate(i-1) 
             list_ptr(j)%m2_4_err = rate_err(i-1) 
             list_ptr(j)%m2_4_exp = exp_map(i-1)
             list_ptr(j)%m2_4_vig = vignetting(i-1)
             list_ptr(j)%m2_4_ml = det_ml(i-1)
          end if
       end if


       if( id_band(i-1) == 5) then
          if( instrument(i-1) == instr_pn) then
             list_ptr(j)%pn_5_flux     = flux(i-1) 
             list_ptr(j)%err_pn_5_flux = flux_err(i-1)
             list_ptr(j)%pn_5     = rate(i-1) 
             list_ptr(j)%pn_5_err = rate_err(i-1)
             list_ptr(j)%pn_5_exp = exp_map(i-1)
             list_ptr(j)%pn_5_vig = vignetting(i-1) 
             list_ptr(j)%pn_5_ml = det_ml(i-1)
          end if
          if( instrument(i-1) == instr_m1) then
             list_ptr(j)%m1_5_flux     = flux(i-1) 
             list_ptr(j)%err_m1_5_flux = flux_err(i-1)
             list_ptr(j)%m1_5     = rate(i-1) 
             list_ptr(j)%m1_5_err = rate_err(i-1) 
             list_ptr(j)%m1_5_exp = exp_map(i-1)
             list_ptr(j)%m1_5_vig = vignetting(i-1) 
             list_ptr(j)%m1_5_ml = det_ml(i-1)
          end if
          if( instrument(i-1) == instr_m2) then
             list_ptr(j)%m2_5_flux     = flux(i-1) 
             list_ptr(j)%err_m2_5_flux = flux_err(i-1)
             list_ptr(j)%m2_5     = rate(i-1) 
             list_ptr(j)%m2_5_err = rate_err(i-1) 
             list_ptr(j)%m2_5_exp = exp_map(i-1)
             list_ptr(j)%m2_5_vig = vignetting(i-1) 
             list_ptr(j)%m2_5_ml = det_ml(i-1)
          end if
       end if

       if( id_band(i-1) == 6) then
          if( instrument(i-1) == instr_pn) then
             list_ptr(j)%pn_6     = rate(i-1) 
             list_ptr(j)%pn_6_err = rate_err(i-1)
             list_ptr(j)%pn_6_exp = exp_map(i-1)
             list_ptr(j)%pn_6_vig = vignetting(i-1) 
             list_ptr(j)%pn_6_ml = det_ml(i-1)
          end if
          if( instrument(i-1) == instr_m1) then
             list_ptr(j)%m1_6     = rate(i-1) 
             list_ptr(j)%m1_6_err = rate_err(i-1) 
             list_ptr(j)%m1_6_exp = exp_map(i-1)
             list_ptr(j)%m1_6_vig = vignetting(i-1) 
             list_ptr(j)%m1_6_ml = det_ml(i-1)
          end if
          if( instrument(i-1) == instr_m2) then
             list_ptr(j)%m2_6     = rate(i-1) 
             list_ptr(j)%m2_6_err = rate_err(i-1) 
             list_ptr(j)%m2_6_exp = exp_map(i-1)
             list_ptr(j)%m2_6_vig = vignetting(i-1) 
             list_ptr(j)%m1_6_ml = det_ml(i-1)
          end if
       end if

       if( (id_band(i-1) == 0 .and. xidlist) .or. (single_band .and. xidlist)) then       
          if( instrument(i-1) == instr_all) then
             list_ptr(j)%ep_xid     = rate(i-1) 
             list_ptr(j)%ep_xid_err = rate_err(i-1)
          end if         
       endif

       if( id_band(i-1) == xid_band) then
          if( instrument(i-1) == instr_all) then
             list_ptr(j)%ep_xid     = rate(i-1) 
             list_ptr(j)%ep_xid_err = rate_err(i-1)
          end if
          if( instrument(i-1) == instr_pn) then
             list_ptr(j)%pn_xid_flux     = flux(i-1) 
             list_ptr(j)%err_pn_xid_flux = flux_err(i-1)
             list_ptr(j)%pn_xid     = rate(i-1) 
             list_ptr(j)%pn_xid_err = rate_err(i-1)
             list_ptr(j)%pn_xid_ml = det_ml(i-1)
          end if
          if( instrument(i-1) == instr_m1) then
             list_ptr(j)%m1_xid_flux     = flux(i-1) 
             list_ptr(j)%err_m1_xid_flux = flux_err(i-1)
             list_ptr(j)%m1_xid     = rate(i-1) 
             list_ptr(j)%m1_xid_err = rate_err(i-1) 
             list_ptr(j)%m1_xid_ml = det_ml(i-1)
          end if
          if( instrument(i-1) == instr_m2) then
             list_ptr(j)%m2_xid_flux     = flux(i-1) 
             list_ptr(j)%err_m2_xid_flux = flux_err(i-1)
             list_ptr(j)%m2_xid     = rate(i-1) 
             list_ptr(j)%m2_xid_err = rate_err(i-1)
             list_ptr(j)%m2_xid_ml = det_ml(i-1) 
          end if
       end if




!       if (j<10) print *,j,list_ptr(j)%ep_ext_ml(1:2),i,ext_ml(i-1),filename(1:20)
     end do

!    print "(A I5 I5)",'Check nsources ',j,n_sources

    do i=1,colnr
       call release( col(i))
    end do

  end function read_set

  !--------------------------------------------------------------------
  subroutine read_Header(set, Header)
  !--------------------------------------------------------------------

    type(DataSetT), intent(in) :: set
    type(Header_type) , intent(out) :: Header

    integer :: i, j
    character(len=1) ::strBand
    character(len=2), dimension(3) :: strInst
    character(len=9) :: strEmin, strEmax

    data strInst /"PN","M1","M2"/
 
    if (hasAttribute(set,"TELESCOP") ) then
       Header%telescop = stringAttribute(set,"TELESCOP")
    else
       Header%telescop = 'UNDEFINED'
    end if
    if( hasAttribute(set,"INSTRUME") ) then
       Header%instrume = stringAttribute(set,"INSTRUME")
    else
       Header%instrume = 'UNDEFINED'
    end if
    if(  hasAttribute(set,"DETNAM") )then
       Header%detnam   = stringAttribute(set,"DETNAM")
    else
       Header%detnam   =  'UNDEFINED'
    end if
    if(  hasAttribute(set,"DATAMODE") ) then
       Header%datamode = stringAttribute(set,"DATAMODE")
    else
       Header%datamode =  'UNDEFINED'
    end if
    if(  hasAttribute(set,"OBS_MODE") ) then
       Header%obs_mode= stringAttribute(set,"OBS_MODE")
    else
       Header%obs_mode =  'UNDEFINED'
    end if
    if(  hasAttribute(set,"FILTER") ) then
       Header%filter = stringAttribute(set,"FILTER")
    else
       Header%filter =  'NONE'
    end if
    if(  hasAttribute(set,"OBSERVER")) then
       Header%observer   = stringAttribute(set,"OBSERVER")
    else
       Header%observer = 'UNDEFINED'
    end if
    if(  hasAttribute(set,"OBS_ID")) then
       Header%obs_id   = stringAttribute(set,"OBS_ID")
    else
       Header%obs_id = 'UNDEFINED'
    end if
    if( hasAttribute(set,"EXP_ID") ) then
       Header%exp_id   = stringAttribute(set,"EXP_ID")
    else
       Header%exp_id   ='UNDEFINED'
    end if
    if( hasAttribute(set,"CONTENT") ) then
       Header%content  = stringAttribute(set,"CONTENT")
    else
       Header%content  ='UNDEFINED'
    end if
    if(  hasAttribute(set,"ORIGIN") ) then
       Header%origin   = stringAttribute(set,"ORIGIN")
    else
       Header%origin   = 'UNDEFINED'
    end if
    if(  hasAttribute(set,"OBJECT")) then
       Header%object   = stringAttribute(set,"OBJECT")
    else
       Header%object   =  'UNDEFINED'
    end if
    if(  hasAttribute(set,"ATT_SRC")) then
       Header%att_src   = stringAttribute(set,"ATT_SRC")
    else
       Header%att_src   =  'UNDEFINED'
    end if

    if(  hasAttribute(set,"DATE-OBS") ) then
       Header%date_obs = stringAttribute(set,"DATE-OBS")
    else
       Header%date_obs = 'UNDEFINED'
    end if
    if(  hasAttribute(set,"DATE-END") ) then
       Header%date_end = stringAttribute(set,"DATE-END")
    else
       Header%date_end = 'UNDEFINED'
    end if
    if(  hasAttribute(set,"RADECSYS") ) then
       Header%radecsys = stringAttribute(set,"RADECSYS")
    else
       Header%radecsys = 'UNDEFINED'
    end if
    if(  hasAttribute(set, "RA_PNT") ) then
       Header%ra_pnt   = real64Attribute(set, "RA_PNT")
    else
       Header%ra_pnt   = 0.0
    end if
    if(  hasAttribute(set, "DEC_PNT") ) then
       Header%dec_pnt  = real64Attribute(set, "DEC_PNT")
    else
       Header%dec_pnt  = 0.0
    end if
    if(  hasAttribute(set, "PA_PNT") ) then
       Header%pa_pnt  = real64Attribute(set, "PA_PNT")
    else
       Header%pa_pnt  = 0.0
    end if
    if(  hasAttribute(set, "EQUINOX")) then
       Header%equinox  = real32Attribute(set, "EQUINOX")
    else
       Header%equinox  = 2000.0
    end if
    if(  hasAttribute(set, "FILT_PN")) then
       Header%filt_pn  = stringAttribute(set, "FILT_PN")
    else
       Header%filt_pn  = 'UNDEFINED'
    end if
    if(  hasAttribute(set, "FILT_M1")) then
       Header%filt_m1  = stringAttribute(set, "FILT_M1")
    else
       Header%filt_m1  = 'UNDEFINED'
    end if 
    if(  hasAttribute(set, "FILT_M2")) then
       Header%filt_m2  = stringAttribute(set, "FILT_M2")
    else
       Header%filt_m2  = 'UNDEFINED'
    end if
    if(  hasAttribute(set, "SUBM_PN")) then
       Header%subm_pn  = stringAttribute(set, "SUBM_PN")
    else
       Header%subm_pn  = 'UNDEFINED'
    end if
    if(  hasAttribute(set, "SUBM_M1")) then
       Header%subm_m1  = stringAttribute(set, "SUBM_M1")
    else
       Header%subm_m1  = 'UNDEFINED'
    end if 
    if(  hasAttribute(set, "SUBM_M2")) then
       Header%subm_m2  = stringAttribute(set, "SUBM_M2")
    else
       Header%subm_m2  = 'UNDEFINED'
    end if
    if(  hasAttribute(set, "EXPOS_PN")) then
       Header%expos_pn  = real32Attribute(set, "EXPOS_PN")
    else
       Header%expos_pn  = 0.0
    end if
    if(  hasAttribute(set, "EXPOS_M1")) then
       Header%expos_m1  = real32Attribute(set, "EXPOS_M1")
    else
       Header%expos_m1  = 0.0
    end if
    if(  hasAttribute(set, "EXPOS_M2")) then
       Header%expos_m2  = real32Attribute(set, "EXPOS_M2")
    else
       Header%expos_m2  = 0.0
    end if
    
!!! IT 2022/12 optionally, read image bin size
    if (skycoord) then
       if      ( hasAttribute(set, "XBINSIZE") ) then
          Header%imbinsize = real32Attribute(set, "XBINSIZE")
       else if ( hasAttribute(set, "YBINSIZE") ) then
          Header%imbinsize = real32Attribute(set, "YBINSIZE")
       else
          call warning("MissingAttribute", &
               "No valid *BINSIZE information available in the source-list &
               &header. Giving extent-related columns in image pixels.")
          skycoord = .false.
       endif
       if ( Header%imbinsize .le. 0) then
          call warning("MissingAttribute", &
               "No valid *BINSIZE information available in the source-list &
               &header. Giving extent-related columns in image pixels.")
          skycoord = .false.
       endif
    endif



!    if(  hasAttribute(set,"EXPOSURE") ) then
!       Header%exposure = real32Attribute(set, "EXPOSURE")
!    else
!       Header%exposure = 0.0
!    end if
    if(  hasAttribute(set,"EXPOSURE") .and. Header%instrume .eq. "EPN     ") then
        expos_pn = expos_pn+real64Attribute(set, "EXPOSURE")
    endif
    if(  hasAttribute(set,"EXPOSURE") .and. Header%instrume .eq. "EMOS1   ") then
       expos_m1 =  expos_m1+real64Attribute(set, "EXPOSURE")
    endif
    if(  hasAttribute(set,"EXPOSURE") .and. Header%instrume .eq. "EMOS2   ") then
       expos_m2 =  expos_m2+real64Attribute(set, "EXPOSURE")
    endif
    if(  hasAttribute(set,"FILTER") .and. Header%instrume .eq. "EPN     ") then
       filt_pn = stringAttribute(set,"FILTER")
    endif
    if(  hasAttribute(set,"FILTER") .and. Header%instrume .eq. "EMOS1   ") then
       filt_m1 = stringAttribute(set,"FILTER")
    endif
    if(  hasAttribute(set,"FILTER") .and. Header%instrume .eq. "EMOS2   ") then
       filt_m2 = stringAttribute(set,"FILTER")
    endif
    if(  hasAttribute(set,"SUBMODE") .and. Header%instrume .eq. "EPN     ") then
       subm_pn = stringAttribute(set,"SUBMODE")
    endif
    if(  hasAttribute(set,"SUBMODE") .and. Header%instrume .eq. "EMOS1   ") then
       subm_m1 = stringAttribute(set,"SUBMODE")
    endif
    if(  hasAttribute(set,"SUBMODE") .and. Header%instrume .eq. "EMOS2   ") then
       subm_m2 = stringAttribute(set,"SUBMODE")
    endif



    do i=1,3
      do j=1,9
        write(strBand,'(i1)') j
        strEmin = strInst(i)//"_"//strBand//"_ELO" 
        strEmax = strInst(i)//"_"//strBand//"_EHI"
        if ( hasAttribute(set, strEmin) ) then
          Header%eblo(i,j) = real32Attribute(set, strEmin)
        else 
          Header%eblo(i,j) = -1.
        endif   
        if ( hasAttribute(set, strEmax) ) then
          Header%ebhi(i,j) = real32Attribute(set, strEmax)
        else 
          Header%ebhi(i,j) = -1.
        endif   
      end do
    end do 

  end subroutine read_Header

  !--------------------------------------------------------------------
  function read_om_set( filename, n_sources ) result( list_ptr)
  !--------------------------------------------------------------------
    type(Source_type), dimension(:), pointer :: list_ptr
    character(*), intent(in) :: filename
    integer(kind=int32) , intent (out) :: n_sources

    type(DataSetT) :: set
    type(TableT)  :: source_tab
    type(ColumnT),dimension(300) :: col

    integer(kind=int32) :: i, nrows, colnr
    integer(kind=int32),dimension(:),pointer :: src_num
    real(kind=double),dimension(:),pointer :: ra, dec
    real(kind=single),dimension(:),pointer :: radec_err



    set = dataSet(filename,READ)
    source_tab = table( set,'SRCLIST')

    !!! IT 2012/07: allow for empty OM source lists - warn and exit
    ! initialize
    n_sources = 0
    colnr=0

    ! get number of rows in this table
    nrows = numberOfRows(source_tab)
    if( nrows <= 0 ) then
       call release(source_tab) ! probably superfluous
       call release(set)        ! probably superfluous
!       call error('emptyInputSet','OM source list is empty')     !!! IT 2012/07
       call warning('emptyInputSet','OM source list is empty')
!    end if                                                       !!! IT 2012/07
    else

    colnr = colnr + 1

    if (hasColumn(source_tab,"SRCNUM")) then
      col(colnr) = column(source_tab, "SRCNUM",READ)
      src_num => int32Data( col(colnr) )
      colnr = colnr + 1
    else
      col(colnr) = column(source_tab, "SRCID",READ)
      src_num => int32Data( col(colnr) )
      colnr = colnr + 1
    end if

    col(colnr) = column(source_tab, "RA",READ)
    ra => real64Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"DEC", READ)
    dec => real64Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"POSERR", READ)
    radec_err => real32Data( col(colnr) )

   ! allocate position list
    allocate(list_ptr(nrows))

    ! Copy the fits data into the structure
    !
    do i=1, nrows
       ! the -1 is the diff. between F90 and C(++)

       n_sources = n_sources + 1
       list_ptr(n_sources)%src_num = src_num(i-1) 
       list_ptr(n_sources)%ra = ra(i-1) 
       list_ptr(n_sources)%dec = dec(i-1)
       list_ptr(n_sources)%radec_err = radec_err(i-1)

    end do

    do i=1,colnr
       call release( col(i))
    end do

    end if
    
  end function read_om_set

  !--------------------------------------------------------------------
  subroutine write_summary_set(filename, srclist, nsrcs, write_ra_corr, &
       header, nsets)
  !--------------------------------------------------------------------
    character(*), intent(in) :: filename
    type(Source_type), dimension(:),intent(in) :: srclist
    integer(kind=int32) , intent (in out) :: nsrcs, nsets
    logical, intent(in) :: write_ra_corr
    type(Header_type), intent(in) :: header

    type(DataSetT) :: set
    type(TableT)  :: tab
    type(ColumnT),dimension(500) :: col    !!! IT: for large stacks (was: 300)

    integer(kind=int32) :: i, j, colnr, ncrossid
    integer(kind=int32),dimension(:),pointer :: src_num, pn_ml_id_src
    integer(kind=int32),dimension(:),pointer :: m1_ml_id_src, m2_ml_id_src
    integer(kind=int32),dimension(:),pointer :: pn_box_id_src
    integer(kind=int32),dimension(:),pointer :: m1_box_id_src, m2_box_id_src
    integer(kind=int32),dimension(:),pointer :: ep_ml_id_src,ep_box_id_src
    real(kind=double), dimension(:), pointer :: ra, dec, bii, lii
    real(kind=double), dimension(:), pointer :: ra_corr, dec_corr
    real(kind=double), dimension(:), pointer :: bii_corr,lii_corr
    real(kind=single), dimension(:), pointer :: radec_err
    real(kind=single), dimension(:), pointer :: pn_tot,pn_tot_err
    real(kind=single), dimension(:), pointer :: m1_tot,m1_tot_err
    real(kind=single), dimension(:), pointer :: m2_tot,m2_tot_err
    real(kind=single), dimension(:), pointer :: pn_scts,pn_scts_err
    real(kind=single), dimension(:), pointer :: m1_scts,m1_scts_err
    real(kind=single), dimension(:), pointer :: m2_scts,m2_scts_err
    real(kind=single), dimension(:), pointer :: ep_tot,ep_tot_err
    !!! IT 2015/07: copy image coordinates (SCR-7301)
    real(kind=single),dimension(:),pointer :: x_ima, x_ima_err, y_ima, y_ima_err
    integer(kind=int32),dimension(:),pointer :: n_opt_co, om_id1
    integer(kind=int32),dimension(:),pointer :: om_id2, om_id3, om_id4,om_id5
    real(kind=single),dimension(:),pointer:: pn_xid, pn_xid_err
    real(kind=single),dimension(:),pointer:: m1_xid, m1_xid_err
    real(kind=single),dimension(:),pointer:: m2_xid, m2_xid_err

    real(kind=single),dimension(:),pointer ::  pn_1_flux, pn_2_flux, pn_3_flux, pn_4_flux, pn_5_flux
    real(kind=single),dimension(:),pointer ::  pn_tot_flux,  pn_xid_flux
    real(kind=single),dimension(:),pointer ::  m1_1_flux, m1_2_flux, m1_3_flux, m1_4_flux, m1_5_flux
    real(kind=single),dimension(:),pointer ::  m1_tot_flux,  m1_xid_flux
    real(kind=single),dimension(:),pointer ::  m2_1_flux, m2_2_flux, m2_3_flux, m2_4_flux, m2_5_flux
    real(kind=single),dimension(:),pointer ::  m2_tot_flux,  m2_xid_flux


    real(kind=single),dimension(:),pointer :: err_pn_1_flux, err_pn_2_flux, err_pn_3_flux, err_pn_4_flux
    real(kind=single),dimension(:),pointer :: err_pn_5_flux  
    real(kind=single),dimension(:),pointer :: err_pn_tot_flux,  err_pn_xid_flux
    real(kind=single),dimension(:),pointer :: err_m1_1_flux, err_m1_2_flux, err_m1_3_flux, err_m1_4_flux
    real(kind=single),dimension(:),pointer :: err_m1_5_flux 
    real(kind=single),dimension(:),pointer :: err_m1_tot_flux,  err_m1_xid_flux
    real(kind=single),dimension(:),pointer :: err_m2_1_flux, err_m2_2_flux, err_m2_3_flux, err_m2_4_flux
    real(kind=single),dimension(:),pointer :: err_m2_5_flux 
    real(kind=single),dimension(:),pointer :: err_m2_tot_flux,  err_m2_xid_flux

    real(kind=single),dimension(:),pointer :: ep_1_flux,err_ep_1_flux, ep_2_flux,err_ep_2_flux
    real(kind=single),dimension(:),pointer :: ep_3_flux,err_ep_3_flux, ep_4_flux,err_ep_4_flux
    real(kind=single),dimension(:),pointer :: ep_5_flux,err_ep_5_flux
    real(kind=single),dimension(:),pointer :: ep_tot_flux,err_ep_tot_flux,ep_xid_flux,err_ep_xid_flux

    real(kind=single),dimension(:),pointer:: pn_1, pn_2, pn_3, pn_4, pn_5
    real(kind=single),dimension(:),pointer:: pn_1_err, pn_2_err, pn_3_err, pn_4_err, pn_5_err
    real(kind=single),dimension(:),pointer:: m1_1, m1_2, m1_3, m1_4, m1_5
    real(kind=single),dimension(:),pointer:: m1_1_err, m1_2_err, m1_3_err, m1_4_err, m1_5_err
    real(kind=single),dimension(:),pointer:: m2_1, m2_2, m2_3, m2_4, m2_5
    real(kind=single),dimension(:),pointer:: m2_1_err, m2_2_err, m2_3_err, m2_4_err, m2_5_err 
    real(kind=single),dimension(:),pointer:: m1_1_exp, m1_2_exp, m1_3_exp, m1_4_exp, m1_5_exp
    real(kind=single),dimension(:),pointer:: m2_1_exp, m2_2_exp, m2_3_exp, m2_4_exp, m2_5_exp
    real(kind=single),dimension(:),pointer:: pn_1_exp, pn_2_exp, pn_3_exp, pn_4_exp, pn_5_exp

    real(kind=single),dimension(:),pointer:: m1_1_ml, m1_2_ml, m1_3_ml, m1_4_ml, m1_5_ml
    real(kind=single),dimension(:),pointer:: m2_1_ml, m2_2_ml, m2_3_ml, m2_4_ml, m2_5_ml
    real(kind=single),dimension(:),pointer:: pn_1_ml, pn_2_ml, pn_3_ml, pn_4_ml, pn_5_ml



    real(kind=single),dimension(:),pointer:: m1_1_vig, m1_2_vig, m1_3_vig, m1_4_vig, m1_5_vig
    real(kind=single),dimension(:),pointer:: m2_1_vig, m2_2_vig, m2_3_vig, m2_4_vig, m2_5_vig
    real(kind=single),dimension(:),pointer:: pn_1_vig, pn_2_vig, pn_3_vig, pn_4_vig, pn_5_vig
    real(kind=single),dimension(:),pointer:: pn_ontime, m1_ontime, m2_ontime
    real(kind=single),dimension(:),pointer:: pn_maskfrac, m1_maskfrac, m2_maskfrac
    real(kind=single),dimension(:),pointer:: ep_xid, ep_xid_err
    real(kind=single),dimension(:),pointer:: pn_det_ml, m1_det_ml, m2_det_ml
    real(kind=single),dimension(:),pointer:: pn_xid_ml, m1_xid_ml, m2_xid_ml
    real(kind=single),dimension(:),pointer:: ep_det_ml, ep_xid_ml
    real(kind=single),dimension(:),pointer:: pn_extent, pn_ext_err,pn_ext_ml
    real(kind=single),dimension(:),pointer:: m1_extent, m1_ext_err,m1_ext_ml
    real(kind=single),dimension(:),pointer:: m2_extent, m2_ext_err,m2_ext_ml
    real(kind=single),dimension(:),pointer:: ep_extent, ep_ext_err,ep_ext_ml
    real(kind=single),dimension(:),pointer:: pn_offax,m1_offax,m2_offax
    real(kind=single),dimension(:),pointer:: pn_hr1,pn_hr1_err
    real(kind=single),dimension(:),pointer:: pn_hr2,pn_hr2_err
    real(kind=single),dimension(:),pointer:: pn_hr3,pn_hr3_err 
    real(kind=single),dimension(:),pointer:: pn_hr4,pn_hr4_err 
    real(kind=single),dimension(:),pointer:: m1_hr1,m1_hr1_err
    real(kind=single),dimension(:),pointer:: m1_hr2,m1_hr2_err 
    real(kind=single),dimension(:),pointer:: m1_hr3,m1_hr3_err 
    real(kind=single),dimension(:),pointer:: m1_hr4,m1_hr4_err 
    real(kind=single),dimension(:),pointer:: m2_hr1,m2_hr1_err
    real(kind=single),dimension(:),pointer:: m2_hr2,m2_hr2_err 
    real(kind=single),dimension(:),pointer:: m2_hr3,m2_hr3_err 
    real(kind=single),dimension(:),pointer:: m2_hr4,m2_hr4_err 
    real(kind=single),dimension(:),pointer:: ep_hr1,ep_hr1_err
    real(kind=single),dimension(:),pointer:: ep_hr2,ep_hr2_err
    real(kind=single),dimension(:),pointer:: ep_hr3,ep_hr3_err
    real(kind=single),dimension(:),pointer:: ep_hr4,ep_hr4_err
    real(kind=single),dimension(:),pointer:: pn_pileup, m1_pileup, m2_pileup
    real(kind=single),dimension(:),pointer:: var_stat, p_var
    integer(kind=int32),dimension(:),pointer :: n_bins, var_exp_no 
    real(kind=single),dimension(:),pointer:: om_id1_dist, om_id2_dist
    real(kind=single),dimension(:),pointer::om_id3_dist,om_id4_dist,om_id5_dist
    real(kind=single) :: pn_ext_ml0,m1_ext_ml0,m2_ext_ml0,ep_ext_ml0  
    real(kind=single) :: ep_tot0, ep_xid0, ep_tot_err0, ep_xid_err0
    real(kind=single) :: pn_det_ml0,m1_det_ml0,m2_det_ml0,ep_det_ml0
    integer(kind=int32) :: nsetss
    integer(kind=int32), dimension(0:3) :: ep
    character(len=1) :: jstring
    character(len=2) :: jjstring       !!! IT 2016/02
    character(len=10) :: colname
    character(len=12) :: extunit       !!! IT 2022
    
!    character(10),dimension(:),pointer :: omflag
!    character(12),dimension(:),pointer :: pn_val_flag
!    character(12),dimension(:),pointer :: pn_ver_flag
!    character(40),dimension(:),pointer :: pn_ver_comm
!    character(12),dimension(:),pointer :: m1_val_flag
!    character(12),dimension(:),pointer :: m1_ver_flag
!    character(40),dimension(:),pointer :: m1_ver_comm
!    character(12),dimension(:),pointer :: m2_val_flag
!    character(12),dimension(:),pointer :: m2_ver_flag
!    character(40),dimension(:),pointer :: m2_ver_comm

    logical(kind=bool),dimension(:),pointer:: tseries, spectra
    logical :: sing_inst

!    allocate(pn_val_flag(nsrcs))
!    allocate(pn_ver_flag(nsrcs))
!    allocate(pn_ver_comm(nsrcs))
!    allocate(m1_val_flag(nsrcs))
!    allocate(m1_ver_flag(nsrcs))
!    allocate(m1_ver_comm(nsrcs))
!    allocate(m2_val_flag(nsrcs))
!    allocate(m2_ver_flag(nsrcs))
!    allocate(m2_ver_comm(nsrcs))


    if( nsrcs <= 0 ) then
       call error('noData','The summary list is empty!')
    else   !!! IT 2016/02
       write(errstr,*) nsrcs, " sources in output table ", filename
      call message(AppMsg,Verbose)
    end if
    set = dataSet(filename,CREATE)
    call write_primary_header(set, header)
    tab = addTable(set,"SRCLIST",nsrcs)
    call write_summary_header(set, tab, header)

!    write(*,*) "write_summ:",srclist(1:100)%ep_ext_ml(2)


!    if (header%ncrossid == 0 .and. nsets <= 3) then
!      sing_inst=.TRUE.
!      ep=0
!      do j=1,nsets
!        ep(header%id_inst(j))=ep(header%id_inst(j))+1
!      end do
!      do i=1,3
!        if (ep(i)>1) sing_inst=.FALSE.
!      end do
!    else
!      sing_inst=.FALSE.
!    end if
! <= replaced by taking single_instr from header written in  read_set (v 3.15.2):
! =>
    sing_inst = header%single_instr
    if (sing_inst) then
      ep=0
      do j=1,nsets
        ep(header%id_inst(j))=ep(header%id_inst(j))+1
      end do
      do i=1,3
        if (ep(i)>1) sing_inst=.FALSE.
      end do
    endif  
!!!!!!!!!!!!!
   
    nsetss=nsets
    if (sing_inst) nsetss=1


    if (skycoord) then              !!! IT 20222
       extunit = "arcsec"
    else
       extunit = "image pixels"
    endif

    
    colnr=0


    colnr = colnr + 1
    col(colnr) = addColumn (tab, "SRC_NUM",Integer32)
    src_num => int32Data( col(colnr) )      

    if (.not. sing_inst) then
!         colnr = colnr + 1
!         col(colnr) = addColumn (tab, "EP_ML_ID_SRC",Integer32," - ",(/nsets/))
!         ep_ml_id_src => int32Data( col(colnr) ) 
!         call setNullValue(col(colnr),-1)
!         call setVColtoNull(col(colnr),nsrcs,nsets)
!         colnr = colnr + 1
!         col(colnr) = addColumn (tab, "EP_BOX_ID_SRC",Integer32," - ",(/nsets/))
!         ep_box_id_src => int32Data( col(colnr) )
!         call setNullValue(col(colnr),-1)
!       call setVColtoNull(col(colnr),nsrcs,nsets)
      do j=1,nsets
        if (j < 10) then
           write(jstring,'(I1.1)') j
           colname="EP_"//jstring//"_ML_ID"
        else
           write(jjstring,'(I2.2)') j
           colname="EP_"//jjstring//"_MLID"   !!! IT 2016/02 - inoff. workaround
        endif
        colnr = colnr + 1
        col(colnr) = addColumn (tab, colname,Integer32," - ")
        ep_ml_id_src => int32Data( col(colnr) )
        call setNullValue(col(colnr),-1)
        call setColtoNull(col(colnr),nsrcs)
        do i=1,nsrcs
             if (srclist(i)%ep_ml_id_src(j)>0) &
                ep_ml_id_src(i-1) = srclist(i)%ep_ml_id_src(j)
        enddo
        colnr = colnr + 1
        if (j < 10) then
           colname="EP_"//jstring//"_BOXID"
        else
           colname="EP_"//jjstring//"_BXID"   !!! IT 2016/02 - inoff. workaround
        endif
        col(colnr) = addColumn (tab, colname,Integer32," - ")
        ep_box_id_src => int32Data( col(colnr) )
        call setNullValue(col(colnr),-1)
        call setColtoNull(col(colnr),nsrcs)
        do i=1,nsrcs
          if (srclist(i)%ep_box_id_src(j)>0) &
              ep_box_id_src(i-1) = srclist(i)%ep_box_id_src(j)
        enddo
      enddo
    end if 

    ncrossid=MAX(header%ncrossid, 1)
  if (sing_inst) then 
    colnr = colnr + 1
    col(colnr)=addColumn(tab,"PN_ML_ID_SRC",Integer32," - ",(/ncrossid/))
    pn_ml_id_src => int32Data( col(colnr) )      
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_ML_ID_SRC",Integer32," - ",(/ncrossid/))
    m1_ml_id_src => int32Data( col(colnr) )      
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_ML_ID_SRC",Integer32," - ",(/ncrossid/))
    m2_ml_id_src => int32Data( col(colnr) )      
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_BOX_ID_SRC",Integer32," - ",(/ncrossid/))
    pn_box_id_src => int32Data( col(colnr) )      
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_BOX_ID_SRC",Integer32," - ",(/ncrossid/))
    m1_box_id_src => int32Data( col(colnr) )
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_BOX_ID_SRC",Integer32," - ",(/ncrossid/))
    m2_box_id_src => int32Data( col(colnr) )
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
  endif
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "RA",Real64, "deg")
    ra => real64Data( col(colnr) )      
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "DEC",Real64, "deg")
    dec => real64Data( col(colnr) )   
    if( write_ra_corr ) then   
       colnr = colnr + 1
       col(colnr) = addColumn (tab, "RA_CORR",Real64, "deg")
       ra_corr => real64Data( col(colnr) )      
       colnr = colnr + 1
       col(colnr) = addColumn (tab, "DEC_CORR",Real64, "deg")
       dec_corr => real64Data( col(colnr) )  
    end if
    colnr = colnr + 1
    col(colnr) = addColumn(tab,"RADEC_ERR",Real32, "arcsec")
    radec_err => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = addColumn(tab,"LII",Real64, "deg")
    lii => real64Data( col(colnr) )            
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "BII",Real64, "deg")
    bii => real64Data( col(colnr) )            
    if( write_ra_corr ) then 
       call setAttribute(tab,"RAOFFSET",header%raoffset,"arcsec",&
            "RA correction applied")
       call setAttribute(tab,"DEOFFSET",header%deoffset,"arcsec",&
            "DEC correction applied")
       call setAttribute(tab,"RAOFFERR",header%raofferr,"arcsec",&
            "Error of RA correction")
       call setAttribute(tab,"DEOFFERR",header%deofferr,"arcsec",&
            "Error of DEC correction")
       call setAttribute(tab,"ROT_CORR",header%rot_corr,"degrees",&
            "Rotation correction applied")
       call setAttribute(tab,"ROT_ERR",header%rot_err,"degrees",&
            "Error in rotation correction")
       call setAttribute(tab,"LIK_HOOD",header%lik_hood,"none",&
            "statistic")
       call setAttribute(tab,"NMATCHES",header%nmatches,"number",&
            "Number of optical-X-ray matches (eposcor)")
       colnr = colnr + 1
       col(colnr) = addColumn(tab,"LII_CORR",Real64, "deg")
       lii_corr => real64Data( col(colnr) )            
       colnr = colnr + 1
       col(colnr) = addColumn (tab, "BII_CORR",Real64, "deg")
       bii_corr => real64Data( col(colnr) )            
    end if
    !!! IT 2015/07: copy image coordinates of one emldetect/eboxdetect input
    !!! list; set them to NULL otherwise copy image coordinates (SCR-7301)
    colnr = colnr + 1
    col(colnr) = addColumn(tab,"X_IMA",Real32, "image pixels")
    x_ima => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn(tab,"X_IMA_ERR",Real32, "image pixels")
    x_ima_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn(tab,"Y_IMA",Real32, "image pixels")
    y_ima => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn(tab,"Y_IMA_ERR",Real32, "image pixels")
    y_ima_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_CTS",Real32,"counts")
    pn_scts => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_CTS_ERR",Real32,"counts")
    pn_scts_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_CTS",Real32,"counts")
    m1_scts => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_CTS_ERR",Real32,"counts")
    m1_scts_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_CTS",Real32,"counts")
    m2_scts => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_CTS_ERR",Real32,"counts")
    m2_scts_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_TOT_FLUX",Real32,"cgs")
    pn_tot_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_PN_TOT_FLUX",Real32,"cgs")
    err_pn_tot_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_TOT_FLUX",Real32,"cgs")
    m1_tot_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_M1_TOT_FLUX",Real32,"cgs")
    err_m1_tot_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_TOT_FLUX",Real32,"cgs")
    m2_tot_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_M2_TOT_FLUX",Real32,"cgs")
    err_m2_tot_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_TOT_FLUX",Real32,"cgs")
    ep_tot_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_EP_TOT_FLUX",Real32,"cgs")
    err_ep_tot_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_1_FLUX",Real32,"cgs")
    pn_1_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_PN_1_FLUX",Real32,"cgs")
    err_pn_1_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_1_FLUX",Real32,"cgs")
    m1_1_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_M1_1_FLUX",Real32,"cgs")
    err_m1_1_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_1_FLUX",Real32,"cgs")
    m2_1_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_M2_1_FLUX",Real32,"cgs")
    err_m2_1_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_1_FLUX",Real32,"cgs")
    ep_1_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_EP_1_FLUX",Real32,"cgs")
    err_ep_1_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_2_FLUX",Real32,"cgs")
    pn_2_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_PN_2_FLUX",Real32,"cgs")
    err_pn_2_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_2_FLUX",Real32,"cgs")
    m1_2_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_M1_2_FLUX",Real32,"cgs")
    err_m1_2_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_2_FLUX",Real32,"cgs")
    m2_2_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_M2_2_FLUX",Real32,"cgs")
    err_m2_2_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_2_FLUX",Real32,"cgs")
    ep_2_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_EP_2_FLUX",Real32,"cgs")
    err_ep_2_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_3_FLUX",Real32,"cgs")
    pn_3_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_PN_3_FLUX",Real32,"cgs")
    err_pn_3_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_3_FLUX",Real32,"cgs")
    m1_3_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_M1_3_FLUX",Real32,"cgs")
    err_m1_3_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_3_FLUX",Real32,"cgs")
    m2_3_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_M2_3_FLUX",Real32,"cgs")
    err_m2_3_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_3_FLUX",Real32,"cgs")
    ep_3_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_EP_3_FLUX",Real32,"cgs")
    err_ep_3_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_4_FLUX",Real32,"cgs")
    pn_4_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_PN_4_FLUX",Real32,"cgs")
    err_pn_4_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_4_FLUX",Real32,"cgs")
    m1_4_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_M1_4_FLUX",Real32,"cgs")
    err_m1_4_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_4_FLUX",Real32,"cgs")
    m2_4_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_M2_4_FLUX",Real32,"cgs")
    err_m2_4_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_4_FLUX",Real32,"cgs")
    ep_4_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_EP_4_FLUX",Real32,"cgs")
    err_ep_4_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_5_FLUX",Real32,"cgs")
    pn_5_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_PN_5_FLUX",Real32,"cgs")
    err_pn_5_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_5_FLUX",Real32,"cgs")
    m1_5_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_M1_5_FLUX",Real32,"cgs")
    err_m1_5_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_5_FLUX",Real32,"cgs")
    m2_5_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_M2_5_FLUX",Real32,"cgs")
    err_m2_5_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_5_FLUX",Real32,"cgs")
    ep_5_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_EP_5_FLUX",Real32,"cgs")
    err_ep_5_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_XID_FLUX",Real32,"cgs")
    pn_xid_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_PN_XID_FLUX",Real32,"cgs")
    err_pn_xid_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_XID_FLUX",Real32,"cgs")
    m1_xid_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_M1_XID_FLUX",Real32,"cgs")
    err_m1_xid_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_XID_FLUX",Real32,"cgs")
    m2_xid_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_M2_XID_FLUX",Real32,"cgs")
    err_m2_xid_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_XID_FLUX",Real32,"cgs")
    ep_xid_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"ERR_EP_XID_FLUX",Real32,"cgs")
    err_ep_xid_flux => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_TOT",Real32,"cnts/s")
    pn_tot => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_TOT_ERR",Real32,"cnts/s")
    pn_tot_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_TOT",Real32,"cnts/s")
    m1_tot => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_TOT_ERR",Real32,"cnts/s")
    m1_tot_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_TOT",Real32,"cnts/s")
    m2_tot => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_TOT_ERR",Real32,"cnts/s")
    m2_tot_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_TOT",Real32,"cnts/s")
    ep_tot => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_TOT_ERR",Real32,"cnts/s")
    ep_tot_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_1",Real32,"cnts/s")
    pn_1 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_1_ERR",Real32,"cnts/s")
    pn_1_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_2",Real32,"cnts/s")
    pn_2 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_2_ERR",Real32,"cnts/s")
    pn_2_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_3",Real32,"cnts/s")
    pn_3 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_3_ERR",Real32,"cnts/s")
    pn_3_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_4",Real32,"cnts/s")
    pn_4 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_4_ERR",Real32,"cnts/s")
    pn_4_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_5",Real32,"cnts/s")
    pn_5 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_5_ERR",Real32,"cnts/s")
    pn_5_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
!    colnr = colnr + 1
!    col(colnr)= addColumn(tab,"PN_6",Real32,"cnts/s")
!    pn_6 => real32Data( col(colnr) )
!    call setColtoNull(col(colnr),nsrcs)
!    colnr = colnr + 1
!    col(colnr)= addColumn(tab,"PN_6_ERR",Real32,"cnts/s")
!    pn_6_err => real32Data( col(colnr) )
!    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_1",Real32,"cnts/s")
    m1_1 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_1_ERR",Real32,"cnts/s")
    m1_1_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_2",Real32,"cnts/s")
    m1_2 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_2_ERR",Real32,"cnts/s")
    m1_2_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_3",Real32,"cnts/s")
    m1_3 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_3_ERR",Real32,"cnts/s")
    m1_3_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_4",Real32,"cnts/s")
    m1_4 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_4_ERR",Real32,"cnts/s")
    m1_4_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_5",Real32,"cnts/s")
    m1_5 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_5_ERR",Real32,"cnts/s")
    m1_5_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
!    colnr = colnr + 1
!    col(colnr)= addColumn(tab,"M1_6",Real32,"cnts/s")
!    m1_6 => real32Data( col(colnr) )
!    call setColtoNull(col(colnr),nsrcs)
!    colnr = colnr + 1
!    col(colnr)= addColumn(tab,"M1_6_ERR",Real32,"cnts/s")
!    m1_6_err => real32Data( col(colnr) )
!    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_1",Real32,"cnts/s")
    m2_1 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_1_ERR",Real32,"cnts/s")
    m2_1_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_2",Real32,"cnts/s")
    m2_2 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_2_ERR",Real32,"cnts/s")
    m2_2_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_3",Real32,"cnts/s")
    m2_3 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_3_ERR",Real32,"cnts/s")
    m2_3_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_4",Real32,"cnts/s")
    m2_4 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_4_ERR",Real32,"cnts/s")
    m2_4_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_5",Real32,"cnts/s")
    m2_5 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_5_ERR",Real32,"cnts/s")
    m2_5_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
!    colnr = colnr + 1
!    col(colnr)= addColumn(tab,"M2_6",Real32,"cnts/s")
!    m2_6 => real32Data( col(colnr) )
!    call setColtoNull(col(colnr),nsrcs)
!    colnr = colnr + 1
!    col(colnr)= addColumn(tab,"M2_6_ERR",Real32,"cnts/s")
!    m2_6_err => real32Data( col(colnr) )
!    call setColtoNull(col(colnr),nsrcs)

 
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_XID",Real32,"cnts/s")
    pn_xid => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_XID_ERR",Real32,"cnts/s")
    pn_xid_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_XID",Real32,"cnts/s")
    m1_xid => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_XID_ERR",Real32,"cnts/s")
    m1_xid_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_XID",Real32,"cnts/s")
    m2_xid => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_XID_ERR",Real32,"cnts/s")
    m2_xid_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_XID",Real32,"cnts/s")
    ep_xid => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_XID_ERR",Real32,"cnts/s")
    ep_xid_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    if (xidflag) then
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "PN_DET_ML",Real32," - ")
      pn_det_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M1_DET_ML",Real32," - ")
      m1_det_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M2_DET_ML",Real32," - ")
      m2_det_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "EP_DET_ML",Real32," - ")
      ep_det_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "PN_XID_ML",Real32," - ")
      pn_xid_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M1_XID_ML",Real32," - ")
      m1_xid_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M2_XID_ML",Real32," - ")
      m2_xid_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "EP_XID_ML",Real32," - ")
      ep_xid_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    

      colnr = colnr + 1
      col(colnr) = addColumn (tab, "PN_1_ML",Real32," - ")
      pn_1_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "PN_2_ML",Real32," - ")
      pn_2_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "PN_3_ML",Real32," - ")
      pn_3_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "PN_4_ML",Real32," - ")
      pn_4_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "PN_5_ML",Real32," - ")
      pn_5_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M1_1_ML",Real32," - ")
      m1_1_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M1_2_ML",Real32," - ")
      m1_2_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M1_3_ML",Real32," - ")
      m1_3_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M1_4_ML",Real32," - ")
      m1_4_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M1_5_ML",Real32," - ")
      m1_5_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M2_1_ML",Real32," - ")
      m2_1_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M2_2_ML",Real32," - ")
      m2_2_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M2_3_ML",Real32," - ")
      m2_3_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M2_4_ML",Real32," - ")
      m2_4_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M2_5_ML",Real32," - ")
      m2_5_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)    
    else
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "PN_DET_ML",Real32," - ",(/nsetss/))
      pn_det_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M1_DET_ML",Real32," - ",(/nsetss/))
      m1_det_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M2_DET_ML",Real32," - ",(/nsetss/))
      m2_det_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "EP_DET_ML",Real32," - ",(/nsetss/))
      ep_det_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "PN_XID_ML",Real32," - ",(/nsetss/))
      pn_xid_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M1_XID_ML",Real32," - ",(/nsetss/))
      m1_xid_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M2_XID_ML",Real32," - ",(/nsetss/))
      m2_xid_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
    endif
    if (.not. allepicflag) then
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "PN_EXTENT",Real32,extunit)
      pn_extent => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "PN_EXT_ERR",Real32,extunit)
      pn_ext_err => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "PN_EXT_ML",Real32,"-",(/nsetss/))
      pn_ext_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M1_EXTENT",Real32,extunit)
      m1_extent => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M1_EXT_ERR",Real32,extunit)
      m1_ext_err => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M1_EXT_ML",Real32,"-",(/nsetss/))
      m1_ext_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M2_EXTENT",Real32,extunit)
      m2_extent => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M2_EXT_ERR",Real32,extunit)
      m2_ext_err => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "M2_EXT_ML",Real32,"-",(/nsetss/))
      m2_ext_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
    endif
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_EXTENT",Real32,extunit)
    ep_extent => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_EXT_ERR",Real32,extunit)
    ep_ext_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    if (xidflag) then
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "EP_EXT_ML",Real32,"-")
      ep_ext_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
    else
      colnr = colnr + 1
      col(colnr) = addColumn (tab, "EP_EXT_ML",Real32,"-",(/nsetss/))
      ep_ext_ml => real32Data( col(colnr) )
      call setColtoNull(col(colnr),nsrcs)
    endif
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_1_EXP",Real32,"seconds")
    pn_1_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_2_EXP",Real32,"seconds")
    pn_2_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_3_EXP",Real32,"seconds")
    pn_3_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_4_EXP",Real32,"seconds")
    pn_4_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_5_EXP",Real32,"seconds")
    pn_5_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_1_VIG",Real32,"-")
    pn_1_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_2_VIG",Real32,"-")
    pn_2_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_3_VIG",Real32,"-")
    pn_3_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_4_VIG",Real32,"-")
    pn_4_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_5_VIG",Real32,"-")
    pn_5_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

!    colnr = colnr + 1
!    col(colnr)= addColumn(tab,"PN_6_EXP",Real32,"seconds")
!    pn_6_exp => real32Data( col(colnr) )
!    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_1_EXP",Real32,"seconds")
    m1_1_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_2_EXP",Real32,"seconds")
    m1_2_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_3_EXP",Real32,"seconds")
    m1_3_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_4_EXP",Real32,"seconds")
    m1_4_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_5_EXP",Real32,"seconds")
    m1_5_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_1_VIG",Real32,"-")
    m1_1_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_2_VIG",Real32,"-")
    m1_2_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_3_VIG",Real32,"-")
    m1_3_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_4_VIG",Real32,"-")
    m1_4_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_5_VIG",Real32,"-")
    m1_5_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)



!    colnr = colnr + 1
!    col(colnr)= addColumn(tab,"M1_6_EXP",Real32,"seconds")
!    m1_6_exp => real32Data( col(colnr) )
!    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_1_EXP",Real32,"seconds")
    m2_1_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_2_EXP",Real32,"seconds")
    m2_2_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_3_EXP",Real32,"seconds")
    m2_3_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_4_EXP",Real32,"seconds")
    m2_4_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_5_EXP",Real32,"seconds")
    m2_5_exp => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
!    colnr = colnr + 1
!    col(colnr)= addColumn(tab,"M2_6_EXP",Real32,"seconds")
!    m2_6_exp => real32Data( col(colnr) )
!    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_1_VIG",Real32,"-")
    m2_1_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_2_VIG",Real32,"-")
    m2_2_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_3_VIG",Real32,"-")
    m2_3_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_4_VIG",Real32,"-")
    m2_4_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_5_VIG",Real32,"-")
    m2_5_vig => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_ONTIME",Real32,"seconds")
    pn_ontime => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_ONTIME",Real32,"seconds")
    m1_ontime => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_ONTIME",Real32,"seconds")
    m2_ontime => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_MASKFRAC",Real32,"-")
    pn_maskfrac => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_MASKFRAC",Real32,"-")
    m1_maskfrac => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_MASKFRAC",Real32,"-")
    m2_maskfrac => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)


    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_OFFAX",Real32,"arcmin")
    pn_offax => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_OFFAX",Real32,"arcmin")
    m1_offax => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_OFFAX",Real32,"arcmin")
    m2_offax => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)


    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_HR1",Real32,"-")
    pn_hr1 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_HR1_ERR",Real32,"-")
    pn_hr1_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_HR2",Real32,"-")
    pn_hr2 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_HR2_ERR",Real32,"-")
    pn_hr2_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_HR3",Real32,"-")
    pn_hr3 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_HR3_ERR",Real32,"-")
    pn_hr3_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_HR4",Real32,"-")
    pn_hr4 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_HR4_ERR",Real32,"-")
    pn_hr4_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_HR1",Real32,"-")
    m1_hr1 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_HR1_ERR",Real32,"-")
    m1_hr1_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_HR2",Real32,"-")
    m1_hr2 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_HR2_ERR",Real32,"-")
    m1_hr2_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_HR3",Real32,"-")
    m1_hr3 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_HR3_ERR",Real32,"-")
    m1_hr3_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_HR4",Real32,"-")
    m1_hr4 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_HR4_ERR",Real32,"-")
    m1_hr4_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_HR1",Real32,"-")
    m2_hr1 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_HR1_ERR",Real32,"-")
    m2_hr1_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_HR2",Real32,"-")
    m2_hr2 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_HR2_ERR",Real32,"-")
    m2_hr2_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_HR3",Real32,"-")
    m2_hr3 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_HR3_ERR",Real32,"-")
    m2_hr3_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_HR4",Real32,"-")
    m2_hr4 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_HR4_ERR",Real32,"-")
    m2_hr4_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_HR1",Real32,"-")
    ep_hr1 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_HR1_ERR",Real32,"-")
    ep_hr1_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_HR2",Real32,"-")
    ep_hr2 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_HR2_ERR",Real32,"-")
    ep_hr2_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_HR3",Real32,"-")
    ep_hr3 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_HR3_ERR",Real32,"-")
    ep_hr3_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)    
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_HR4",Real32,"-")
    ep_hr4 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_HR4_ERR",Real32,"-")
    ep_hr4_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_PILEUP",Real32,"-")
    pn_pileup => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_PILEUP",Real32,"-")
    m1_pileup => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_PILEUP",Real32,"-")
    m2_pileup => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"VAR_STAT",Real32)
    var_stat => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"N_BINS",Integer32)
    n_bins  => int32Data(col(colnr) ) 
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"P_VAR",Real32)
    p_var => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"VAR_EXP_NO",Integer32)
    var_exp_no  => int32Data(col(colnr) ) 
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)

    !
    ! OM COLUMNS:
    !
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID1",Integer32)
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    om_id1 => int32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID1_DIST",Real32,"arcsec")
    om_id1_dist => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID2",Integer32)
    om_id2 => int32Data( col(colnr) )
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID2_DIST",Real32,"arcsec")
    om_id2_dist => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID3",Integer32)
    om_id3 => int32Data( col(colnr) )
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID3_DIST",Real32,"arcsec")
    om_id3_dist => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID4",Integer32)
    om_id4 => int32Data( col(colnr) )
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID4_DIST",Real32,"arcsec")
    om_id4_dist => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID5",Integer32)
    om_id5 => int32Data( col(colnr) )
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID5_DIST",Real32,"arcsec")
    om_id5_dist => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"N_OPT_CO",Integer32)
    n_opt_co => int32Data( col(colnr) )
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)




    do i=1,nsrcs
       src_num(i-1) = srclist(i)%src_num

       if( header%ncrossid >= 1 ) then
          do j=1,header%ncrossid
             if (srclist(i)%pn_ml_id_src(j)>0) &
               pn_ml_id_src( (i-1)*ncrossid + j-1) = srclist(i)%pn_ml_id_src(j)
             if (srclist(i)%m1_ml_id_src(j)>0) &
               m1_ml_id_src( (i-1)*ncrossid + j-1) = srclist(i)%m1_ml_id_src(j)
             if (srclist(i)%m2_ml_id_src(j)>0) &
               m2_ml_id_src( (i-1)*ncrossid + j-1) = srclist(i)%m2_ml_id_src(j)
             if (srclist(i)%pn_box_id_src(j)>0) &
               pn_box_id_src( (i-1)*ncrossid + j-1) = srclist(i)%pn_box_id_src(j)
             if (srclist(i)%m1_box_id_src(j)>0) &
               m1_box_id_src( (i-1)*ncrossid + j-1) = srclist(i)%m1_box_id_src(j)
             if (srclist(i)%m2_box_id_src(j)>0) &
               m2_box_id_src( (i-1)*ncrossid + j-1) = srclist(i)%m2_box_id_src(j)

          end do

       else
          !pn_ml_id_src(i-1) = null_entry
          !m1_ml_id_src(i-1) = null_entry
          !m2_ml_id_src(i-1) = null_entry
          !pn_box_id_src(i-1) =null_entry
          !m1_box_id_src(i-1) =null_entry
          !m2_box_id_src(i-1) =null_entry
       end if

       if (sing_inst) then
          do j=1,nsets
             if (header%id_inst(j) == 1) then
             if (srclist(i)%ep_ml_id_src(j)>0) &
                pn_ml_id_src(i-1) = srclist(i)%ep_ml_id_src(j)
             if (srclist(i)%ep_box_id_src(j)>0) &
                pn_box_id_src(i-1) = srclist(i)%ep_box_id_src(j)
             end if
             if (header%id_inst(j) == 2) then
             if (srclist(i)%ep_ml_id_src(j)>0) &
                m1_ml_id_src(i-1) = srclist(i)%ep_ml_id_src(j)
             if (srclist(i)%ep_box_id_src(j)>0) &
                m1_box_id_src(i-1) = srclist(i)%ep_box_id_src(j)
             end if
             if (header%id_inst(j) == 3) then
             if (srclist(i)%ep_ml_id_src(j)>0) &
                m2_ml_id_src(i-1) = srclist(i)%ep_ml_id_src(j)
             if (srclist(i)%ep_box_id_src(j)>0) &
                m2_box_id_src(i-1) = srclist(i)%ep_box_id_src(j)
             end if
          end do         
       else     
!          do j=1,nsets
!             if (srclist(i)%ep_ml_id_src(j)>0) &
!                ep_ml_id_src( (i-1)*nsets + j-1) = srclist(i)%ep_ml_id_src(j)
!             if (srclist(i)%ep_box_id_src(j)>0) &
!                ep_box_id_src( (i-1)*nsets + j-1) = srclist(i)%ep_box_id_src(j)
!          end do
       end if


       ra(i-1)        = srclist(i)%ra
       dec(i-1)       = srclist(i)%dec
       if( write_ra_corr ) then 
          ra_corr(i-1)   = srclist(i)%ra_corr
          dec_corr(i-1)  = srclist(i)%dec_corr
          lii_corr(i-1)    = srclist(i)%lii_corr
          bii_corr(i-1)    = srclist(i)%bii_corr
       end if
       radec_err(i-1) = srclist(i)%radec_err
       lii(i-1)       = srclist(i)%lii
       bii(i-1)       = srclist(i)%bii
       !!! IT 2015/07: copy image coordinates of one emldetect/eboxdetect input
       !!! list; set them to NULL otherwise  (SCR-7301)
       if (nsetss==1) x_ima(i-1)     = srclist(i)%x_ima
       if (nsetss==1) x_ima_err(i-1) = srclist(i)%x_ima_err
       if (nsetss==1) y_ima(i-1)     = srclist(i)%y_ima
       if (nsetss==1) y_ima_err(i-1) = srclist(i)%y_ima_err




       if (srclist(i)%pn_tot>-1e30) & 
          pn_tot(i-1)      =  srclist(i)%pn_tot
       if (srclist(i)%pn_tot_err>-1e30) & 
          pn_tot_err(i-1)  =  srclist(i)%pn_tot_err
       if (srclist(i)%m1_tot>-1e30) & 
          m1_tot(i-1)      =  srclist(i)%m1_tot
       if (srclist(i)%m1_tot_err>-1e30) & 
          m1_tot_err(i-1)  =  srclist(i)%m1_tot_err
       if (srclist(i)%m2_tot>-1e30) & 
          m2_tot(i-1)      =  srclist(i)%m2_tot
       if (srclist(i)%m2_tot_err>-1e30) & 
          m2_tot_err(i-1)  =  srclist(i)%m2_tot_err
       if (srclist(i)%pn_scts>-1e30) & 
          pn_scts(i-1)      =  srclist(i)%pn_scts
       if (srclist(i)%pn_scts_err>-1e30) & 
          pn_scts_err(i-1)  =  srclist(i)%pn_scts_err
       if (srclist(i)%m1_scts>-1e30) & 
          m1_scts(i-1)      =  srclist(i)%m1_scts
       if (srclist(i)%m1_scts_err>-1e30) & 
          m1_scts_err(i-1)  =  srclist(i)%m1_scts_err
       if (srclist(i)%m2_scts>-1e30) & 
          m2_scts(i-1)      =  srclist(i)%m2_scts
       if (srclist(i)%m2_scts_err>-1e30) & 
          m2_scts_err(i-1)  =  srclist(i)%m2_scts_err


       if (sing_inst) then
         ep_tot0=0. 
         ep_tot_err0=0.
         if (srclist(i)%pn_tot>-1e30) &
            ep_tot0 = ep_tot0 + srclist(i)%pn_tot
         if (srclist(i)%pn_tot_err>-1e30) &
            ep_tot_err0 = ep_tot_err0 + srclist(i)%pn_tot_err**2 
         if (srclist(i)%m1_tot>-1e30) &
            ep_tot0 = ep_tot0 + srclist(i)%m1_tot
         if (srclist(i)%m1_tot_err>-1e30) &
            ep_tot_err0 = ep_tot_err0 + srclist(i)%m1_tot_err**2 
         if (srclist(i)%m2_tot>-1e30) &
            ep_tot0 = ep_tot0 + srclist(i)%m2_tot
         if (srclist(i)%m2_tot_err>-1e30) &
            ep_tot_err0 = ep_tot_err0 + srclist(i)%m2_tot_err**2 
         if (ep_tot0>0.) then
           ep_tot(i-1) = ep_tot0
           ep_tot_err(i-1) = sqrt(ep_tot_err0)
          end if
       else
         ep_tot(i-1)      =  srclist(i)%ep_tot
         ep_tot_err(i-1)  =  srclist(i)%ep_tot_err
       end if

       if (srclist(i)%pn_1>-1e30) & 
          pn_1(i-1)      =  srclist(i)%pn_1
       if (srclist(i)%pn_1_err>-1e30) & 
          pn_1_err(i-1)  =  srclist(i)%pn_1_err
       if (srclist(i)%m1_1>-1e30) &  
          m1_1(i-1)      =  srclist(i)%m1_1
       if (srclist(i)%m1_1_err>-1e30) & 
          m1_1_err(i-1)  =  srclist(i)%m1_1_err
       if (srclist(i)%m2_1>-1e30) & 
          m2_1(i-1)      =  srclist(i)%m2_1
       if (srclist(i)%m2_1_err>-1e30) & 
          m2_1_err(i-1)  =  srclist(i)%m2_1_err

       if (srclist(i)%pn_2>-1e30) & 
          pn_2(i-1)      =  srclist(i)%pn_2
       if (srclist(i)%pn_2_err>-1e30) & 
          pn_2_err(i-1)  =  srclist(i)%pn_2_err
       if (srclist(i)%m1_2>-1e30) &  
          m1_2(i-1)      =  srclist(i)%m1_2
       if (srclist(i)%m1_2_err>-1e30) & 
          m1_2_err(i-1)  =  srclist(i)%m1_2_err
       if (srclist(i)%m2_2>-1e30) & 
          m2_2(i-1)      =  srclist(i)%m2_2
       if (srclist(i)%m2_2_err>-1e30) & 
          m2_2_err(i-1)  =  srclist(i)%m2_2_err

       if (srclist(i)%pn_3>-1e30) & 
          pn_3(i-1)      =  srclist(i)%pn_3
       if (srclist(i)%pn_3_err>-1e30) & 
          pn_3_err(i-1)  =  srclist(i)%pn_3_err
       if (srclist(i)%m1_3>-1e30) &  
          m1_3(i-1)      =  srclist(i)%m1_3
       if (srclist(i)%m1_3_err>-1e30) & 
          m1_3_err(i-1)  =  srclist(i)%m1_3_err
       if (srclist(i)%m2_3>-1e30) & 
          m2_3(i-1)      =  srclist(i)%m2_3
       if (srclist(i)%m2_3_err>-1e30) & 
          m2_3_err(i-1)  =  srclist(i)%m2_3_err

       if (srclist(i)%pn_4>-1e30) & 
          pn_4(i-1)      =  srclist(i)%pn_4
       if (srclist(i)%pn_4_err>-1e30) & 
          pn_4_err(i-1)  =  srclist(i)%pn_4_err
       if (srclist(i)%m1_4>-1e30) &  
          m1_4(i-1)      =  srclist(i)%m1_4
       if (srclist(i)%m1_4_err>-1e30) & 
          m1_4_err(i-1)  =  srclist(i)%m1_4_err
       if (srclist(i)%m2_4>-1e30) & 
          m2_4(i-1)      =  srclist(i)%m2_4
       if (srclist(i)%m2_4_err>-1e30) & 
          m2_4_err(i-1)  =  srclist(i)%m2_4_err

       if (srclist(i)%pn_5>-1e30) & 
          pn_5(i-1)      =  srclist(i)%pn_5
       if (srclist(i)%pn_5_err>-1e30) & 
          pn_5_err(i-1)  =  srclist(i)%pn_5_err
       if (srclist(i)%m1_5>-1e30) &  
          m1_5(i-1)      =  srclist(i)%m1_5
       if (srclist(i)%m1_5_err>-1e30) & 
          m1_5_err(i-1)  =  srclist(i)%m1_5_err
       if (srclist(i)%m2_5>-1e30) & 
          m2_5(i-1)      =  srclist(i)%m2_5
       if (srclist(i)%m2_5_err>-1e30) & 
          m2_5_err(i-1)  =  srclist(i)%m2_5_err

!       if (srclist(i)%pn_6>-1e30) & 
!          pn_6(i-1)      =  srclist(i)%pn_6
!       if (srclist(i)%pn_6_err>-1e30) & 
!          pn_6_err(i-1)  =  srclist(i)%pn_6_err
!       if (srclist(i)%m1_6>-1e30) &  
!          m1_6(i-1)      =  srclist(i)%m1_6
!       if (srclist(i)%m1_6_err>-1e30) & 
!          m1_6_err(i-1)  =  srclist(i)%m1_6_err
!       if (srclist(i)%m2_6>-1e30) & 
!          m2_6(i-1)      =  srclist(i)%m2_6
!       if (srclist(i)%m2_6_err>-1e30) & 
!          m2_6_err(i-1)  =  srclist(i)%m2_6_err

       if (xidflag) then
         ! individual likelihoods are only written in XID mode
         if (srclist(i)%pn_1_ml>-1e30) & 
            pn_1_ml(i-1)      =  srclist(i)%pn_1_ml
         if (srclist(i)%pn_2_ml>-1e30) & 
            pn_2_ml(i-1)      =  srclist(i)%pn_2_ml
         if (srclist(i)%pn_3_ml>-1e30) & 
            pn_3_ml(i-1)      =  srclist(i)%pn_3_ml
         if (srclist(i)%pn_4_ml>-1e30) & 
            pn_4_ml(i-1)      =  srclist(i)%pn_4_ml
         if (srclist(i)%pn_5_ml>-1e30) & 
            pn_5_ml(i-1)      =  srclist(i)%pn_5_ml
         if (srclist(i)%m1_1_ml>-1e30) & 
            m1_1_ml(i-1)      =  srclist(i)%m1_1_ml
         if (srclist(i)%m1_2_ml>-1e30) & 
            m1_2_ml(i-1)      =  srclist(i)%m1_2_ml
         if (srclist(i)%m1_3_ml>-1e30) & 
            m1_3_ml(i-1)      =  srclist(i)%m1_3_ml
         if (srclist(i)%m1_4_ml>-1e30) & 
            m1_4_ml(i-1)      =  srclist(i)%m1_4_ml
         if (srclist(i)%m1_5_ml>-1e30) & 
            m1_5_ml(i-1)      =  srclist(i)%m1_5_ml
         if (srclist(i)%m2_1_ml>-1e30) & 
            m2_1_ml(i-1)      =  srclist(i)%m2_1_ml
         if (srclist(i)%m2_2_ml>-1e30) & 
            m2_2_ml(i-1)      =  srclist(i)%m2_2_ml
         if (srclist(i)%m2_3_ml>-1e30) & 
            m2_3_ml(i-1)      =  srclist(i)%m2_3_ml
         if (srclist(i)%m2_4_ml>-1e30) & 
            m2_4_ml(i-1)      =  srclist(i)%m2_4_ml
         if (srclist(i)%m2_5_ml>-1e30) & 
            m2_5_ml(i-1)      =  srclist(i)%m2_5_ml
       endif

       if (srclist(i)%pn_1_exp>-1e30) & 
          pn_1_exp(i-1)      =  srclist(i)%pn_1_exp
       if (srclist(i)%pn_2_exp>-1e30) & 
          pn_2_exp(i-1)      =  srclist(i)%pn_2_exp
       if (srclist(i)%pn_3_exp>-1e30) & 
          pn_3_exp(i-1)      =  srclist(i)%pn_3_exp
       if (srclist(i)%pn_4_exp>-1e30) & 
          pn_4_exp(i-1)      =  srclist(i)%pn_4_exp
       if (srclist(i)%pn_5_exp>-1e30) & 
          pn_5_exp(i-1)      =  srclist(i)%pn_5_exp
!       if (srclist(i)%pn_6_exp>-1e30) & 
!          pn_6_exp(i-1)      =  srclist(i)%pn_6_exp
       if (srclist(i)%m1_1_exp>-1e30) & 
          m1_1_exp(i-1)      =  srclist(i)%m1_1_exp
       if (srclist(i)%m1_2_exp>-1e30) & 
          m1_2_exp(i-1)      =  srclist(i)%m1_2_exp
       if (srclist(i)%m1_3_exp>-1e30) & 
          m1_3_exp(i-1)      =  srclist(i)%m1_3_exp
       if (srclist(i)%m1_4_exp>-1e30) & 
          m1_4_exp(i-1)      =  srclist(i)%m1_4_exp
       if (srclist(i)%m1_5_exp>-1e30) & 
          m1_5_exp(i-1)      =  srclist(i)%m1_5_exp
!       if (srclist(i)%m1_6_exp>-1e30) & 
!          m1_6_exp(i-1)      =  srclist(i)%m1_6_exp
       if (srclist(i)%m2_1_exp>-1e30) & 
          m2_1_exp(i-1)      =  srclist(i)%m2_1_exp
       if (srclist(i)%m2_2_exp>-1e30) & 
          m2_2_exp(i-1)      =  srclist(i)%m2_2_exp
       if (srclist(i)%m2_3_exp>-1e30) & 
          m2_3_exp(i-1)      =  srclist(i)%m2_3_exp
       if (srclist(i)%m2_4_exp>-1e30) & 
          m2_4_exp(i-1)      =  srclist(i)%m2_4_exp
       if (srclist(i)%m2_5_exp>-1e30) & 
          m2_5_exp(i-1)      =  srclist(i)%m2_5_exp
!       if (srclist(i)%m2_6_exp>-1e30) & 
!          m2_6_exp(i-1)      =  srclist(i)%m2_6_exp


       if (srclist(i)%pn_1_vig>-1e30) & 
          pn_1_vig(i-1)      =  srclist(i)%pn_1_vig
       if (srclist(i)%pn_2_vig>-1e30) & 
          pn_2_vig(i-1)      =  srclist(i)%pn_2_vig
       if (srclist(i)%pn_3_vig>-1e30) & 
          pn_3_vig(i-1)      =  srclist(i)%pn_3_vig
       if (srclist(i)%pn_4_vig>-1e30) & 
          pn_4_vig(i-1)      =  srclist(i)%pn_4_vig
       if (srclist(i)%pn_5_vig>-1e30) & 
          pn_5_vig(i-1)      =  srclist(i)%pn_5_vig
       if (srclist(i)%m1_1_vig>-1e30) & 
          m1_1_vig(i-1)      =  srclist(i)%m1_1_vig
       if (srclist(i)%m1_2_vig>-1e30) & 
          m1_2_vig(i-1)      =  srclist(i)%m1_2_vig
       if (srclist(i)%m1_3_vig>-1e30) & 
          m1_3_vig(i-1)      =  srclist(i)%m1_3_vig
       if (srclist(i)%m1_4_vig>-1e30) & 
          m1_4_vig(i-1)      =  srclist(i)%m1_4_vig
       if (srclist(i)%m1_5_vig>-1e30) & 
          m1_5_vig(i-1)      =  srclist(i)%m1_5_vig
       if (srclist(i)%m2_1_vig>-1e30) & 
          m2_1_vig(i-1)      =  srclist(i)%m2_1_vig
       if (srclist(i)%m2_2_vig>-1e30) & 
          m2_2_vig(i-1)      =  srclist(i)%m2_2_vig
       if (srclist(i)%m2_3_vig>-1e30) & 
          m2_3_vig(i-1)      =  srclist(i)%m2_3_vig
       if (srclist(i)%m2_4_vig>-1e30) & 
          m2_4_vig(i-1)      =  srclist(i)%m2_4_vig
       if (srclist(i)%m2_5_vig>-1e30) & 
          m2_5_vig(i-1)      =  srclist(i)%m2_5_vig
       if (srclist(i)%pn_ontime>-1e30) & 
          pn_ontime(i-1)      =  srclist(i)%pn_ontime
       if (srclist(i)%m1_ontime>-1e30) & 
          m1_ontime(i-1)      =  srclist(i)%m1_ontime
       if (srclist(i)%m2_ontime>-1e30) & 
          m2_ontime(i-1)      =  srclist(i)%m2_ontime
       if (srclist(i)%pn_maskfrac>-1e30) & 
          pn_maskfrac(i-1)      =  srclist(i)%pn_maskfrac
       if (srclist(i)%m1_maskfrac>-1e30) & 
          m1_maskfrac(i-1)      =  srclist(i)%m1_maskfrac
       if (srclist(i)%m2_maskfrac>-1e30) & 
          m2_maskfrac(i-1)      =  srclist(i)%m2_maskfrac
       if (srclist(i)%pn_xid>-1e30) & 
          pn_xid(i-1)      =  srclist(i)%pn_xid
       if (srclist(i)%pn_xid_err>-1e30) & 
          pn_xid_err(i-1)  =  srclist(i)%pn_xid_err
       if (srclist(i)%m1_xid>-1e30) &  
          m1_xid(i-1)      =  srclist(i)%m1_xid
       if (srclist(i)%m1_xid_err>-1e30) & 
          m1_xid_err(i-1)  =  srclist(i)%m1_xid_err
       if (srclist(i)%m2_xid>-1e30) & 
          m2_xid(i-1)      =  srclist(i)%m2_xid
       if (srclist(i)%m2_xid_err>-1e30) & 
          m2_xid_err(i-1)  =  srclist(i)%m2_xid_err
       if (sing_inst) then
         ep_xid0=0.
         ep_xid_err0=0.
         if (srclist(i)%pn_xid>-1e30) &
            ep_xid0 = ep_xid0 + srclist(i)%pn_xid
         if (srclist(i)%pn_xid_err>-1e30) &
            ep_xid_err0 = ep_xid_err0 + srclist(i)%pn_xid_err**2
         if (srclist(i)%m1_xid>-1e30) &
            ep_xid0 = ep_xid0 + srclist(i)%m1_xid
         if (srclist(i)%m1_xid_err>-1e30) &
            ep_xid_err0 = ep_xid_err0 + srclist(i)%m1_xid_err**2
         if (srclist(i)%m2_xid>-1e30) &
            ep_xid0 = ep_xid0 + srclist(i)%m2_xid
         if (srclist(i)%m2_xid_err>-1e30) &
            ep_xid_err0 = ep_xid_err0 + srclist(i)%m2_xid_err**2
         if (ep_xid0>0.) then
            ep_xid(i-1) = ep_xid0
            ep_xid_err(i-1) = sqrt(ep_xid_err0)
         end if
       else if (xidflag) then
         if (srclist(i)%ep_xid>-1e30) &
            ep_xid(i-1)      =  srclist(i)%ep_xid
         if (srclist(i)%ep_xid_err>-1e30) &
            ep_xid_err(i-1)  =  srclist(i)%ep_xid_err
       else
         if (srclist(i)%ep_xid>-1e30) &
            ep_xid(i-1)      =  srclist(i)%ep_xid
         if (srclist(i)%ep_xid_err>-1e30) &
            ep_xid_err(i-1)  =  srclist(i)%ep_xid_err
       end if
       if (sing_inst) then
          pn_det_ml0 = 0.
          m1_det_ml0 = 0.
          m2_det_ml0 = 0.
          ep_det_ml0 = 0.
          do j=1,nsets
             pn_det_ml0 = pn_det_ml0 + srclist(i)%pn_det_ml(j)
             m1_det_ml0 = m1_det_ml0 + srclist(i)%m1_det_ml(j)
             m2_det_ml0 = m2_det_ml0 + srclist(i)%m2_det_ml(j)
             ep_det_ml0 = ep_det_ml0 + srclist(i)%pn_det_ml(j) &
                                             + srclist(i)%m1_det_ml(j) &  
                                             + srclist(i)%m2_det_ml(j)
          end do 
          if (pn_det_ml0>0) pn_det_ml(i-1) = pn_det_ml0      
          if (m1_det_ml0>0) m1_det_ml(i-1) = m1_det_ml0      
          if (m2_det_ml0>0) m2_det_ml(i-1) = m2_det_ml0      
          if (ep_det_ml0>0) ep_det_ml(i-1) = ep_det_ml0      
       else if (xidflag) then
             if (srclist(i)%pn_det_ml(1)>-1e30) &
                pn_det_ml(i-1) = srclist(i)%pn_det_ml(1)
             if (srclist(i)%m1_det_ml(1)>-1e30) &
                m1_det_ml(i-1) = srclist(i)%m1_det_ml(1)
             if (srclist(i)%m2_det_ml(1)>-1e30) &
                m2_det_ml(i-1) = srclist(i)%m2_det_ml(1)
             if (srclist(i)%ep_det_ml(1)>-1e30) &
                ep_det_ml(i-1) = srclist(i)%ep_det_ml(1)
             if (srclist(i)%pn_xid_ml(2)>-1e30) &
                pn_xid_ml(i-1) = srclist(i)%pn_xid_ml(2)
             if (srclist(i)%m1_xid_ml(2)>-1e30) &
                m1_xid_ml(i-1) = srclist(i)%m1_xid_ml(2)
             if (srclist(i)%m2_xid_ml(2)>-1e30) &
                m2_xid_ml(i-1) = srclist(i)%m2_xid_ml(2) 
             if (srclist(i)%ep_xid_ml(2)>-1e30) &
                ep_xid_ml(i-1) = srclist(i)%ep_xid_ml(2)        
       else
          do j=1,nsets
             if (srclist(i)%pn_det_ml(j)>-1e30) &
                pn_det_ml( (i-1)*nsets + j-1) = srclist(i)%pn_det_ml(j)
             if (srclist(i)%m1_det_ml(j)>-1e30) &
                m1_det_ml( (i-1)*nsets + j-1) = srclist(i)%m1_det_ml(j)
             if (srclist(i)%m2_det_ml(j)>-1e30) &
                m2_det_ml( (i-1)*nsets + j-1) = srclist(i)%m2_det_ml(j)
             if (srclist(i)%ep_det_ml(j)>-1e30) &
                ep_det_ml( (i-1)*nsets + j-1) = srclist(i)%ep_det_ml(j)
             if (srclist(i)%pn_xid_ml(j)>-1e30) &
                pn_xid_ml( (i-1)*nsets + j-1) = srclist(i)%pn_xid_ml(j)
             if (srclist(i)%m1_xid_ml(j)>-1e30) &
                m1_xid_ml( (i-1)*nsets + j-1) = srclist(i)%m1_xid_ml(j)
             if (srclist(i)%m2_xid_ml(j)>-1e30) &
                m2_xid_ml( (i-1)*nsets + j-1) = srclist(i)%m2_xid_ml(j)
          end do
       end if
       if (.not. allepicflag) then
         if (srclist(i)%pn_extent>-1e30) &
            pn_extent(i-1)   =  srclist(i)%pn_extent
         if (srclist(i)%pn_ext_err>-1e30) &
            pn_ext_err(i-1)  =  srclist(i)%pn_ext_err
         if (srclist(i)%m1_extent>-1e30) &
            m1_extent(i-1)   =  srclist(i)%m1_extent
         if (srclist(i)%m1_ext_err>-1e30) &
            m1_ext_err(i-1)  =  srclist(i)%m1_ext_err
         if (srclist(i)%m2_extent>-1e30) &
            m2_extent(i-1)   =  srclist(i)%m2_extent
         if (srclist(i)%m2_ext_err>-1e30) &
            m2_ext_err(i-1)  =  srclist(i)%m2_ext_err
       endif
       if (srclist(i)%ep_extent>-1e30) &
          ep_extent(i-1)   =  srclist(i)%ep_extent
       if (srclist(i)%ep_ext_err>-1e30) &
          ep_ext_err(i-1)  =  srclist(i)%ep_ext_err
       if (sing_inst .and. .not. allepicflag) then
          pn_ext_ml0 = 0.
          m1_ext_ml0 = 0.
          m2_ext_ml0 = 0.
          ep_ext_ml0 = 0.
          do j=1,nsets
              if (srclist(i)%pn_ext_ml(j)>-1e30) &
               pn_ext_ml0 = pn_ext_ml0 + srclist(i)%pn_ext_ml(j)
             if (srclist(i)%m1_ext_ml(j)>-1e30) &
               m1_ext_ml0 = m1_ext_ml0 + srclist(i)%m1_ext_ml(j)
             if (srclist(i)%m2_ext_ml(j)>-1e30) &
               m2_ext_ml0 = m2_ext_ml0 + srclist(i)%m2_ext_ml(j)
             if (srclist(i)%pn_ext_ml(j)>-1e30) &
               ep_ext_ml0 = ep_ext_ml0 + srclist(i)%pn_ext_ml(j)
             if (srclist(i)%m1_ext_ml(j)>-1e30) &
               ep_ext_ml0 = ep_ext_ml0 + srclist(i)%m1_ext_ml(j)
             if (srclist(i)%m2_ext_ml(j)>-1e30) &
               ep_ext_ml0 = ep_ext_ml0 + srclist(i)%m2_ext_ml(j)
          end do
          if (pn_ext_ml0>0.) pn_ext_ml(i-1)=pn_ext_ml0
          if (m1_ext_ml0>0.) m1_ext_ml(i-1)=m1_ext_ml0       
          if (m2_ext_ml0>0.) m2_ext_ml(i-1)=m2_ext_ml0  
          if (ep_ext_ml0>0.) ep_ext_ml(i-1)=ep_ext_ml0   
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx    
       else if (allepicflag .or. xidflag) then 
          if (srclist(i)%ep_ext_ml(1)>-1e30) then
            ep_ext_ml(i-1) = srclist(i)%ep_ext_ml(1)
          else
            ep_ext_ml(i-1) = null_entry 
          endif
       else      
          do j=1,nsets
             if (srclist(i)%pn_ext_ml(j)>-1e30) &
                pn_ext_ml( (i-1)*nsets + j-1) = srclist(i)%pn_ext_ml(j)
             if (srclist(i)%m1_ext_ml(j)>-1e30) &
                m1_ext_ml( (i-1)*nsets + j-1) = srclist(i)%m1_ext_ml(j)
             if (srclist(i)%m2_ext_ml(j)>-1e30) &
                m2_ext_ml( (i-1)*nsets + j-1) = srclist(i)%m2_ext_ml(j)
             if (srclist(i)%ep_ext_ml(j)>-1e30) then
                ep_ext_ml( (i-1)*nsets + j-1) = srclist(i)%ep_ext_ml(j)
             else
!                ep_ext_ml( (i-1)*nsets + j-1) = null_entry
             endif
          end do
       end if  

       
       if (srclist(i)%pn_tot_flux>-1e30)       pn_tot_flux(i-1) = srclist(i)%pn_tot_flux
       if (srclist(i)%err_pn_tot_flux>-1e30)   err_pn_tot_flux(i-1) = srclist(i)%err_pn_tot_flux
       if (srclist(i)%m1_tot_flux>-1e30)       m1_tot_flux(i-1) = srclist(i)%m1_tot_flux
       if (srclist(i)%err_m1_tot_flux>-1e30)   err_m1_tot_flux(i-1) = srclist(i)%err_m1_tot_flux
       if (srclist(i)%m2_tot_flux>-1e30)       m2_tot_flux(i-1) = srclist(i)%m2_tot_flux
       if (srclist(i)%err_m2_tot_flux>-1e30)   err_m2_tot_flux(i-1) = srclist(i)%err_m2_tot_flux
       if (srclist(i)%ep_tot_flux>-1e30)       ep_tot_flux(i-1) = srclist(i)%ep_tot_flux
       if (srclist(i)%err_ep_tot_flux>-1e30)   err_ep_tot_flux(i-1) = srclist(i)%err_ep_tot_flux

       if (srclist(i)%pn_1_flux>-1e30)       pn_1_flux(i-1) = srclist(i)%pn_1_flux
       if (srclist(i)%err_pn_1_flux>-1e30)   err_pn_1_flux(i-1) = srclist(i)%err_pn_1_flux
       if (srclist(i)%m1_1_flux>-1e30)       m1_1_flux(i-1) = srclist(i)%m1_1_flux
       if (srclist(i)%err_m1_1_flux>-1e30)   err_m1_1_flux(i-1) = srclist(i)%err_m1_1_flux
       if (srclist(i)%m2_1_flux>-1e30)       m2_1_flux(i-1) = srclist(i)%m2_1_flux
       if (srclist(i)%err_m2_1_flux>-1e30)   err_m2_1_flux(i-1) = srclist(i)%err_m2_1_flux
       if (srclist(i)%ep_1_flux>-1e30)       ep_1_flux(i-1) = srclist(i)%ep_1_flux
       if (srclist(i)%err_ep_1_flux>-1e30)   err_ep_1_flux(i-1) = srclist(i)%err_ep_1_flux
       
       if (srclist(i)%pn_2_flux>-1e30)       pn_2_flux(i-1) = srclist(i)%pn_2_flux
       if (srclist(i)%err_pn_2_flux>-1e30)   err_pn_2_flux(i-1) = srclist(i)%err_pn_2_flux
       if (srclist(i)%m1_2_flux>-1e30)       m1_2_flux(i-1) = srclist(i)%m1_2_flux
       if (srclist(i)%err_m1_2_flux>-1e30)   err_m1_2_flux(i-1) = srclist(i)%err_m1_2_flux
       if (srclist(i)%m2_2_flux>-1e30)       m2_2_flux(i-1) = srclist(i)%m2_2_flux
       if (srclist(i)%err_m2_2_flux>-1e30)   err_m2_2_flux(i-1) = srclist(i)%err_m2_2_flux
       if (srclist(i)%ep_2_flux>-1e30)       ep_2_flux(i-1) = srclist(i)%ep_2_flux
       if (srclist(i)%err_ep_2_flux>-1e30)   err_ep_2_flux(i-1) = srclist(i)%err_ep_2_flux
       
       if (srclist(i)%pn_3_flux>-1e30)       pn_3_flux(i-1) = srclist(i)%pn_3_flux
       if (srclist(i)%err_pn_3_flux>-1e30)   err_pn_3_flux(i-1) = srclist(i)%err_pn_3_flux
       if (srclist(i)%m1_3_flux>-1e30)       m1_3_flux(i-1) = srclist(i)%m1_3_flux
       if (srclist(i)%err_m1_3_flux>-1e30)   err_m1_3_flux(i-1) = srclist(i)%err_m1_3_flux
       if (srclist(i)%m2_3_flux>-1e30)       m2_3_flux(i-1) = srclist(i)%m2_3_flux
       if (srclist(i)%err_m2_3_flux>-1e30)   err_m2_3_flux(i-1) = srclist(i)%err_m2_3_flux
       if (srclist(i)%ep_3_flux>-1e30)       ep_3_flux(i-1) = srclist(i)%ep_3_flux
       if (srclist(i)%err_ep_3_flux>-1e30)   err_ep_3_flux(i-1) = srclist(i)%err_ep_3_flux
       
       if (srclist(i)%pn_4_flux>-1e30)       pn_4_flux(i-1) = srclist(i)%pn_4_flux
       if (srclist(i)%err_pn_4_flux>-1e30)   err_pn_4_flux(i-1) = srclist(i)%err_pn_4_flux
       if (srclist(i)%m1_4_flux>-1e30)       m1_4_flux(i-1) = srclist(i)%m1_4_flux
       if (srclist(i)%err_m1_4_flux>-1e30)   err_m1_4_flux(i-1) = srclist(i)%err_m1_4_flux
       if (srclist(i)%m2_4_flux>-1e30)       m2_4_flux(i-1) = srclist(i)%m2_4_flux
       if (srclist(i)%err_m2_4_flux>-1e30)   err_m2_4_flux(i-1) = srclist(i)%err_m2_4_flux
       if (srclist(i)%ep_4_flux>-1e30)       ep_4_flux(i-1) = srclist(i)%ep_4_flux
       if (srclist(i)%err_ep_4_flux>-1e30)   err_ep_4_flux(i-1) = srclist(i)%err_ep_4_flux
       
       if (srclist(i)%pn_5_flux>-1e30)       pn_5_flux(i-1) = srclist(i)%pn_5_flux
       if (srclist(i)%err_pn_5_flux>-1e30)   err_pn_5_flux(i-1) = srclist(i)%err_pn_5_flux
       if (srclist(i)%m1_5_flux>-1e30)       m1_5_flux(i-1) = srclist(i)%m1_5_flux
       if (srclist(i)%err_m1_5_flux>-1e30)   err_m1_5_flux(i-1) = srclist(i)%err_m1_5_flux
       if (srclist(i)%m2_5_flux>-1e30)       m2_5_flux(i-1) = srclist(i)%m2_5_flux
       if (srclist(i)%err_m2_5_flux>-1e30)   err_m2_5_flux(i-1) = srclist(i)%err_m2_5_flux
       if (srclist(i)%ep_5_flux>-1e30)       ep_5_flux(i-1) = srclist(i)%ep_5_flux
       if (srclist(i)%err_ep_5_flux>-1e30)   err_ep_5_flux(i-1) = srclist(i)%err_ep_5_flux
       
       if (srclist(i)%pn_xid_flux>-1e30)       pn_xid_flux(i-1) = srclist(i)%pn_xid_flux
       if (srclist(i)%err_pn_xid_flux>-1e30)   err_pn_xid_flux(i-1) = srclist(i)%err_pn_xid_flux
       if (srclist(i)%m1_xid_flux>-1e30)       m1_xid_flux(i-1) = srclist(i)%m1_xid_flux
       if (srclist(i)%err_m1_xid_flux>-1e30)   err_m1_xid_flux(i-1) = srclist(i)%err_m1_xid_flux
       if (srclist(i)%m2_xid_flux>-1e30)       m2_xid_flux(i-1) = srclist(i)%m2_xid_flux
       if (srclist(i)%err_m2_xid_flux>-1e30)   err_m2_xid_flux(i-1) = srclist(i)%err_m2_xid_flux
       if (srclist(i)%ep_xid_flux>-1e30)       ep_xid_flux(i-1) = srclist(i)%ep_xid_flux
       if (srclist(i)%err_ep_xid_flux>-1e30)   err_ep_xid_flux(i-1) = srclist(i)%err_ep_xid_flux

       if (srclist(i)%pn_offax>-1e30)     pn_offax(i-1)     = srclist(i)%pn_offax 
       if (srclist(i)%m1_offax>-1e30)     m1_offax(i-1)     = srclist(i)%m1_offax 
       if (srclist(i)%m2_offax>-1e30)     m2_offax(i-1)     = srclist(i)%m2_offax 

       if (srclist(i)%pn_hr1>-1e30)     pn_hr1(i-1)     = srclist(i)%pn_hr1
       if (srclist(i)%pn_hr1_err>-1e30) pn_hr1_err(i-1) = srclist(i)%pn_hr1_err
       if (srclist(i)%pn_hr2>-1e30)     pn_hr2(i-1)     = srclist(i)%pn_hr2
       if (srclist(i)%pn_hr2_err>-1e30) pn_hr2_err(i-1) = srclist(i)%pn_hr2_err
       if (srclist(i)%pn_hr3>-1e30)     pn_hr3(i-1)     = srclist(i)%pn_hr3
       if (srclist(i)%pn_hr3_err>-1e30) pn_hr3_err(i-1) = srclist(i)%pn_hr3_err
       if (srclist(i)%pn_hr4>-1e30) pn_hr4(i-1) = srclist(i)%pn_hr4
       if (srclist(i)%pn_hr4_err>-1e30) pn_hr4_err(i-1) = srclist(i)%pn_hr4_err
       if (srclist(i)%m1_hr1>-1e30)     m1_hr1(i-1)     = srclist(i)%m1_hr1
       if (srclist(i)%m1_hr1_err>-1e30) m1_hr1_err(i-1) = srclist(i)%m1_hr1_err
       if (srclist(i)%m1_hr2>-1e30)     m1_hr2(i-1)     = srclist(i)%m1_hr2
       if (srclist(i)%m1_hr2_err>-1e30) m1_hr2_err(i-1) = srclist(i)%m1_hr2_err
       if (srclist(i)%m1_hr3>-1e30)     m1_hr3(i-1)     = srclist(i)%m1_hr3
       if (srclist(i)%m1_hr3_err>-1e30) m1_hr3_err(i-1) = srclist(i)%m1_hr3_err
       if (srclist(i)%m1_hr4>-1e30)     m1_hr4(i-1)     = srclist(i)%m1_hr4
       if (srclist(i)%m1_hr4_err>-1e30) m1_hr4_err(i-1) = srclist(i)%m1_hr4_err
       if (srclist(i)%m2_hr1>-1e30)     m2_hr1(i-1)     = srclist(i)%m2_hr1
       if (srclist(i)%m2_hr1_err>-1e30) m2_hr1_err(i-1) = srclist(i)%m2_hr1_err
       if (srclist(i)%m2_hr2>-1e30)     m2_hr2(i-1)     = srclist(i)%m2_hr2
       if (srclist(i)%m2_hr2_err>-1e30) m2_hr2_err(i-1) = srclist(i)%m2_hr2_err
       if (srclist(i)%m2_hr3>-1e30)     m2_hr3(i-1)     = srclist(i)%m2_hr3
       if (srclist(i)%m2_hr3_err>-1e30) m2_hr3_err(i-1) = srclist(i)%m2_hr3_err
       if (srclist(i)%m2_hr4>-1e30)     m2_hr4(i-1)     = srclist(i)%m2_hr4
       if (srclist(i)%m2_hr4_err>-1e30) m2_hr4_err(i-1) = srclist(i)%m2_hr4_err
       if (srclist(i)%ep_hr1>-1e30)     ep_hr1(i-1)     = srclist(i)%ep_hr1
       if (srclist(i)%ep_hr1_err>-1e30) ep_hr1_err(i-1) = srclist(i)%ep_hr1_err
       if (srclist(i)%ep_hr2>-1e30)     ep_hr2(i-1)     = srclist(i)%ep_hr2 
       if (srclist(i)%ep_hr2_err>-1e30) ep_hr2_err(i-1) = srclist(i)%ep_hr2_err
       if (srclist(i)%ep_hr3>-1e30)     ep_hr3(i-1)     = srclist(i)%ep_hr3
       if (srclist(i)%ep_hr3_err>-1e30) ep_hr3_err(i-1) = srclist(i)%ep_hr3_err
       if (srclist(i)%ep_hr4>-1e30)     ep_hr4(i-1)     = srclist(i)%ep_hr4 
       if (srclist(i)%ep_hr4_err>-1e30) ep_hr4_err(i-1) = srclist(i)%ep_hr4_err

       if (srclist(i)%pn_pileup>-1e30) pn_pileup(i-1) = srclist(i)%pn_pileup
       if (srclist(i)%m1_pileup>-1e30) m1_pileup(i-1) = srclist(i)%m1_pileup
       if (srclist(i)%m2_pileup>-1e30) m2_pileup(i-1) = srclist(i)%m2_pileup

       ! var_stat         = 0.0
       ! p_var            = 0.0
       ! n_bins(i-1)      = 0
       ! var_exp_no       = 0
       if (srclist(i)%om_id(1)>0) &
          om_id1(i-1)      =  srclist(i)%om_id(1)
       if (srclist(i)%om_id_dist(1)>-1e30) &
          om_id1_dist(i-1) =  srclist(i)%om_id_dist(1)
       if (srclist(i)%om_id(2)>0) &
          om_id2(i-1)      =  srclist(i)%om_id(2)
       if (srclist(i)%om_id_dist(2)>-1e30) &
          om_id2_dist(i-1) =  srclist(i)%om_id_dist(2)
       if (srclist(i)%om_id(3)>0) &
          om_id3(i-1)      =  srclist(i)%om_id(3)
       if (srclist(i)%om_id_dist(3)>-1e30) &
          om_id3_dist(i-1) =  srclist(i)%om_id_dist(3)
       if (srclist(i)%om_id(4)>0) &
          om_id4(i-1)      =  srclist(i)%om_id(4)
       if (srclist(i)%om_id_dist(4)>-1e30) &
          om_id4_dist(i-1) =  srclist(i)%om_id_dist(4)
       if (srclist(i)%om_id(5)>0) &
          om_id5(i-1)      =  srclist(i)%om_id(5)
       if (srclist(i)%om_id_dist(5)>-1e30) &
          om_id5_dist(i-1) =  srclist(i)%om_id_dist(5)
       if (srclist(i)%n_opt_co>0) &
          n_opt_co(i-1)    =  srclist(i)%n_opt_co
    end do


    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OMFLAG",String," ",(/10/))
!    omflag => stringData( col(colnr))
    do i=1,nsrcs
       call setStringCell(col(colnr), i-1,  srclist(i)%omflag);
    end do

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"SPECTRA",Boolean)
    spectra => boolData( col(colnr))
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"TSERIES",Boolean)
    tseries => boolData( col(colnr))
    do i=1,nsrcs
       spectra(i-1)= .FALSE.
       tseries(i-1)= .FALSE.
    end do
    
      colnr = colnr + 1
      col(colnr)= addColumn(tab,"PN_FLAG",String," ",(/12/))
      do i=1,nsrcs
           call setStringCell(col(colnr), i-1,srclist(i)%pn_val_flag);
      end do
!      colnr = colnr + 1
!      col(colnr)= addColumn(tab,"PN_VER_FLAG",String," ",(/12/))
!      do i=1,nsrcs
!          call setStringCell(col(colnr), i-1,srclist(i)%pn_ver_flag);
!      end do
!      colnr = colnr + 1
!      col(colnr)= addColumn(tab,"PN_VER_COMM",String," ",(/40/))
!      do i=1,nsrcs
!          call setStringCell(col(colnr), i-1,srclist(i)%pn_ver_comm);
!      end do
      colnr = colnr + 1
      col(colnr)= addColumn(tab,"M1_FLAG",String," ",(/12/))
      do i=1,nsrcs
          call setStringCell(col(colnr), i-1,srclist(i)%m1_val_flag);
      end do
!      colnr = colnr + 1
!      col(colnr)= addColumn(tab,"M1_VER_FLAG",String," ",(/12/))
!      do i=1,nsrcs
!          call setStringCell(col(colnr), i-1,srclist(i)%m1_ver_flag);
!      end do
!      colnr = colnr + 1
!      col(colnr)= addColumn(tab,"M1_VER_COMM",String," ",(/40/))
!      do i=1,nsrcs
!           call setStringCell(col(colnr), i-1,srclist(i)%m1_ver_comm);
!      end do
      colnr = colnr + 1
      col(colnr)= addColumn(tab,"M2_FLAG",String," ",(/12/))
      do i=1,nsrcs
          call setStringCell(col(colnr), i-1,srclist(i)%m2_val_flag);
      end do
      if (allepicflag) then
        colnr = colnr + 1
        col(colnr)= addColumn(tab,"EP_FLAG",String," ",(/12/))
        do i=1,nsrcs
          call setStringCell(col(colnr), i-1,srclist(i)%ep_flag);
        end do 
      endif       
!      colnr = colnr + 1
!      col(colnr)= addColumn(tab,"M2_VER_FLAG",String," ",(/12/))
!      do i=1,nsrcs
!          call setStringCell(col(colnr), i-1,srclist(i)%m2_ver_flag);
!      end do
!      colnr = colnr + 1
!      col(colnr)= addColumn(tab,"M2_VER_COMM",String," ",(/40/))
!      do i=1,nsrcs
!          call setStringCell(col(colnr), i-1,srclist(i)%m2_ver_comm);
!      end do

    do i=1, colnr
       call release(col(i))
    end do

    call release(tab)
    call release(set)

  end subroutine write_summary_set

  !---------------------------------------------------------------------
  subroutine setColtoNull (col, nsrcs)
  !---------------------------------------------------------------------

     type(ColumnT), intent(in) :: col 
     integer, intent(in) :: nsrcs
     integer :: i

     do i=0,nsrcs-1
        call settoNull(col,i)     
     end do

  end subroutine setColtoNull 


  !---------------------------------------------------------------------
  subroutine setVColtoNull (col, nsrcs, npos)
  !---------------------------------------------------------------------

     type(ColumnT), intent(in) :: col 
     integer, intent(in) :: nsrcs, npos
     integer :: i,pos
     
     do pos=0,npos-1
       do i=0,nsrcs-1
         call settoNull(col,i,pos)     
       end do
     end do

  end subroutine setVColtoNull 





  !--------------------------------------------------------------------
  subroutine write_primary_header(set, header)
  !--------------------------------------------------------------------  
    type(DataSetT), intent(in out) :: set
    type(Header_type) , intent(in) :: header

    call setAttribute(set,"TELESCOP", header%telescop,"XMM mission")
    call setAttribute(set,'INSTRUME', "EPIC","EPIC Instrument")
!    call setAttribute(set,'DETNAM', header%detnam)
    call setAttribute(set,'DATAMODE', header%datamode,"Instrument mode (IMAGING or TIMING)")
    call setAttribute(set,'OBS_MODE', header%obs_mode,"Observation mode (pointing or slew)")
    call setAttribute(set,'OBS_ID', header%obs_id,"Observation Identifier")
    call setAttribute(set,'EXP_ID', header%exp_id,"Exposure Identifier")
    call setAttribute(set,'OBJECT', header%object,"Name of observed object")
    call setAttribute(set,'RA_PNT', header%ra_pnt,"deg","Mean RA of the optical axis")
    call setAttribute(set,'DEC_PNT', header%dec_pnt,"deg","Mean Dec of the optical axis")
    call setAttribute(set,'PA_PNT', header%pa_pnt,"deg","Actual (mean) measured position angle")
    call setAttribute(set,'RADECSYS', header%radecsys,"World coord. system for this file")
    call setAttribute(set,'EQUINOX', header%equinox," ","Equinox for sky coordinate x/y axes")
!    call setAttribute(set,'EXPOSURE', header%exposure)
    call setAttribute(set,'EXPOS_PN', expos_pn,"sec","sum of EPN exposure values")
    call setAttribute(set,'EXPOS_M1', expos_m1,"sec","sum of EMOS1 exposure values")
    call setAttribute(set,'EXPOS_M2', expos_m2,"sec","sum of EMOS2 exposure values")
    call setAttribute(set,'DATE-OBS', header%date_obs,"Start time of exposure")
    call setAttribute(set,'DATE-END', header%date_end,"End time of exposure")
    call setAttribute(set,'OBSERVER', header%observer,"Name of observer")
    call setAttribute(set,'ATT_SRC', header%att_src,"Source of attitude data (AHF|RAF|THF)")
    
    if (allepicflag) then
      call setAttribute(set,'FILT_PN', header%filt_pn,"EPN Filter ID")
      call setAttribute(set,'FILT_M1', header%filt_m1,"EMOS1 Filter ID")
      call setAttribute(set,'FILT_M2', header%filt_m2,"EMOS2 Filter ID")
      call setAttribute(set,'SUBM_PN', header%subm_pn,"EPN Guest observer mode")
      call setAttribute(set,'SUBM_M1', header%subm_m1,"EMOS1 Guest observer mode")
      call setAttribute(set,'SUBM_M2', header%subm_m2,"EMOS2 Guest observer mode") 
      call setAttribute(set,'EXPOS_PN', header%expos_pn,"sec","EPN exposure")
      call setAttribute(set,'EXPOS_M1', header%expos_m1,"sec","EMOS1 exposure")
      call setAttribute(set,'EXPOS_M2', header%expos_m2,"sec","EMOS2 exposure")
    else
      call setAttribute(set,'FILT_PN', filt_pn,"EPN Filter ID")
      call setAttribute(set,'FILT_M1', filt_m1,"EMOS1 Filter ID")
      call setAttribute(set,'FILT_M2', filt_m2,"EMOS2 Filter ID")
      call setAttribute(set,'SUBM_PN', subm_pn,"EPN Guest observer mode")
      call setAttribute(set,'SUBM_M1', subm_m1,"EMOS1 Guest observer mode")
      call setAttribute(set,'SUBM_M2', subm_m2,"EMOS2 Guest observer mode")
      call setAttribute(set,'EXPOS_PN', expos_pn,"sec","sum of EPN exposure values")
      call setAttribute(set,'EXPOS_M1', expos_m1,"sec","sum of EMOS1 exposure values")
      call setAttribute(set,'EXPOS_M2', expos_m2,"sec","sum of EMOS2 exposure values")
    endif
    

  end subroutine write_primary_header

  !--------------------------------------------------------------------
  subroutine write_summary_header(set,tab, header)
  !--------------------------------------------------------------------  
    type(DataSetT), intent(in out) :: set
    type(TableT)  :: tab
    type(Header_type) , intent(in) :: header


    character(6) :: setname='EP_SET'
    character(7) :: crossidname='CROSSID'
    character(15) :: set_id 
    character(len=1) ::strBand
    character(len=2), dimension(3) :: strInst
    character(len=9) :: strEmin, strEmax
    character(len=200) :: keyComment
    integer(kind=int32)  :: i,j

    data strInst /"PN","M1","M2"/

    call setAttribute(set,"CONTENT", "EPIC CAMERA ML SOURCE LIST")

    call setAttribute(tab,'NSETS',header%nsets," ",&
         "Number of input data sets.")
    do i=1,header%nsets
       if( i <= 9) then
         write (set_id,'(A6,I1)') setname,i
         write (keyComment,'("srcmatch input list ",I1)') i
       endif
       if( i > 9) then
         write (set_id,'(A6,I2)') setname,i
         write (keyComment,'("srcmatch input list ",I2)') i
       endif
       call setAttribute(tab,set_id,header%inputsets(i) ,keyComment )
    end do
    call setAttribute(tab,'NCROSSID', header%ncrossid," ","Number of cross id sets (earlier stage products).")
    do i=1,header%ncrossid
       write (set_id,'(A7,I1)') crossidname,i
       call setAttribute(tab,set_id, header%crossidsets(i))
    end do

    call setAttribute(tab,"SYST_ERR", header%syst_err,&
         "arcsec","Systematic error for EPIC source matching")
    call setAttribute(tab,"MAX_ERR", header%max_err,"sigmas",&
         "Maximum error for correlating sources")
    call setAttribute(tab,"OMCROSS", header%omcross,&
         "Are X-ray  cross correlated with OM sources?")
    call setAttribute(tab,"OM_DIST", header%om_dist,"arcsec",&
         "Maximum distance for OM sources")

    do i=1,3
      do j=1,9
        if (header%eblo(i,j) >0.) then
          write(strBand,'(i1)') j
          strEmin = strInst(i)//"_"//strBand//"_ELO" 
          call setAttribute(tab,strEmin,header%eblo(i,j))
        end if
        if (header%ebhi(i,j) >0.) then
          write(strBand,'(i1)') j
          strEmax = strInst(i)//"_"//strBand//"_EHI" 
          call setAttribute(tab,strEmax,header%ebhi(i,j))
        end if
      end do
    end do 
    
  end subroutine write_summary_header


  !--------------------------------------------------------------------
  subroutine write_html_list(filename,srclist, nsrcs, header, nsets)
  !--------------------------------------------------------------------  
! D.M. 
! D.M. Changed to reduce the html table massively (by ~ 75% - 80%) to just give
! D.M. an overview of the identified sources, for details see FITS table !
! D.M. 
    character(*), intent(in) :: filename
    type(Source_type), dimension(:),intent(inout) :: srclist
    integer(kind=int32) , intent (in) :: nsrcs, nsets
    type(Header_type) , intent(in) :: header
    integer(kind=int32) :: i,j,ip,left
!    character(35) :: begcell='<TD ALIGN=middle><FONT SIZE=-3>'
!    character(25) :: endcell='</FONT></TD>'
!    character(5) :: seperator=' , '
    character(10) :: forma
    character(1000) :: line

    real(kind=single) :: ep_tot0, ep_tot_err0, &
                         ep_xid0, ep_xid_err0, ep_ext_ml0
    integer(kind=int32), dimension(0:3) :: ep
    logical :: sing_inst


    open(unit=7,file=filename,form="formatted",status="unknown")
    
    if (header%ncrossid == 0 .and. nsets <= 3) then
      sing_inst=.TRUE.
      ep=0
      do j=1,nsets
        ep(header%id_inst(j))=ep(header%id_inst(j))+1
      end do
      do i=1,3
        if (ep(i)>1) sing_inst=.FALSE.
      end do
    else
      sing_inst=.FALSE.
    end if

    write (7,"(a)") "<HTML><HEAD>"
    write (7,"(a)") &
     "<TITLE>EPIC HTML summary source list</TITLE>"
    write (7,"(a)") "</head>"
    write (7,"(a)") "<BODY><CENTER>"
    write (7,"(a)") "<H2>Final EPIC observation source list</H2>"

    write (7,"(a)") "<PRE>"

    do ip=1,200
      line(ip:ip)=" "
    end do 
    ip=1 


    write (line(ip:),"(a)") "  SRC_NUM "
    ip=ip+10
    if (.not. sing_inst) then 
       write (line(ip:),"(a)") " EP_ML_ID " 
       ip=ip+10*header%nsets
       write (line(ip:),"(a)") " EP_BOX_ID " 
       ip=ip+10*header%nsets
    else      
       write (line(ip:),"(3a)") " PN_ML_ID "," M1_ML_ID "," M2_ML_ID " 
       ip=ip+30
       write (line(ip:),"(3a)") " PN_BOX_ID"," M1_BOX_ID"," M2_BOX_ID"
       ip=ip+30

    end if
    write(line(ip:),"(3a)")"    RA    ","    DEC   "," RADEC_ERR"
    ip=ip+30 

    ip=ip+20 

    write(line(ip:),"(2a)")"  EP_XID  ","EP_XID_ERR"
    ip=ip+20 

    if (sing_inst) then
      write(line(ip:),"(3a)")"  EP_EXT  ","EP_EXT_ERR"," EP_EXT_ML"
      ip=ip+30
    else
      write(line(ip:),"(3a)")"  EP_EXT  ","EP_EXT_ERR"
      ip=ip+20
      write(line(ip:),"(3a)")" EP_EXT_ML"
      ip=ip+10*header%nsets
    end if      

    write (line(ip:),"(a)")"  OMFLAG  "
    ip=ip+10

     write (7,"(a,/)")line(1:ip)
    do i=1,nsrcs
    
 
      do ip=1,200
        line(ip:ip)=" "
      end do 
      ip=1 

       left = int(log10(real(i)))+1
       write (forma,"(a,i1,a)") "(a,i",left,",a)" 
       write (line(ip:),forma) '<A NAME="SRC_',i,'">'
       ip=ip+15+left

       write(line(ip:),"(i7,3x)")i
       ip=ip+10


       if (sing_inst) then
         do j=1,nsets
            if (header%id_inst(j) == 1) then
            if (srclist(i)%ep_ml_id_src(j)>0) &
               srclist(i)%pn_ml_id_src(1) = srclist(i)%ep_ml_id_src(j)
            if (srclist(i)%ep_box_id_src(j)>0) &
               srclist(i)%pn_box_id_src(1) = srclist(i)%ep_box_id_src(j)
            end if
            if (header%id_inst(j) == 2) then
            if (srclist(i)%ep_ml_id_src(j)>0) &
               srclist(i)%m1_ml_id_src(1) = srclist(i)%ep_ml_id_src(j)
            if (srclist(i)%ep_box_id_src(j)>0) &
               srclist(i)%m1_box_id_src(1) = srclist(i)%ep_box_id_src(j)
            end if
            if (header%id_inst(j) == 3) then
            if (srclist(i)%ep_ml_id_src(j)>0) &
               srclist(i)%m2_ml_id_src(1) = srclist(i)%ep_ml_id_src(j)
            if (srclist(i)%ep_box_id_src(j)>0) &
               srclist(i)%m2_box_id_src(1) = srclist(i)%ep_box_id_src(j)
            end if
          end do
          if (srclist(i)%pn_ml_id_src(1)>0) then         
            write(line(ip:),"(i7,3x)")srclist(i)%pn_ml_id_src(1)
            ip=ip+10
          else
            write(line(ip:),"(a)")"      -   "
            ip=ip+10
          end if
          if (srclist(i)%m1_ml_id_src(1)>0) then 
            write(line(ip:),"(i7,3x)")srclist(i)%m1_ml_id_src(1)
            ip=ip+10
          else
            write(line(ip:),"(a)")"      -   "
            ip=ip+10
          end if
          if (srclist(i)%m2_ml_id_src(1)>0) then     
            write(line(ip:),"(i7,3x)")srclist(i)%m2_ml_id_src(1)
            ip=ip+10
          else
            write(line(ip:),"(a)")"      -   "
            ip=ip+10
          end if
          if (srclist(i)%pn_box_id_src(1)>0) then 
            write(line(ip:),"(i7,3x)")srclist(i)%pn_box_id_src(1)
            ip=ip+10
          else
            write(line(ip:),"(a)")"      -   "
            ip=ip+10
          end if
          if (srclist(i)%m1_box_id_src(1)>0) then
            write(line(ip:),"(i7,3x)")srclist(i)%m1_box_id_src(1)
            ip=ip+10
          else
            write(line(ip:),"(a)")"      -   "
            ip=ip+10
          end if
          if (srclist(i)%m2_box_id_src(1)>0) then
            write(line(ip:),"(i7,3x)")srclist(i)%m2_box_id_src(1)
            ip=ip+10
          else
            write(line(ip:),"(a)")"      -   "
            ip=ip+10
          end if
       else
          do j=1,header%nsets
             write(line(ip:),"(i7,3x)") srclist(i)%ep_ml_id_src(j)
             ip=ip+10
          end do
          do j=1,header%nsets
             write(line(ip:),"(i7,3x)") srclist(i)%ep_box_id_src(j)
             ip=ip+10
          end do
       end if
       write (line(ip:),"(2f10.5)")srclist(i)%ra,srclist(i)%dec
       ip=ip+20
       write (line(ip:),"(f8.2,2x)")srclist(i)%radec_err
       ip=ip+10
       if (sing_inst) then
          ep_tot0=0. 
          ep_tot_err0=0.
          if (srclist(i)%pn_tot>-1e30) &
             ep_tot0 = ep_tot0 + srclist(i)%pn_tot
          if (srclist(i)%pn_tot_err>-1e30) &
             ep_tot_err0 = ep_tot_err0 + srclist(i)%pn_tot_err**2 
          if (srclist(i)%m1_tot>-1e30) &
             ep_tot0 = ep_tot0 + srclist(i)%m1_tot
          if (srclist(i)%m1_tot_err>-1e30) &
             ep_tot_err0 = ep_tot_err0 + srclist(i)%m1_tot_err**2 
          if (srclist(i)%m2_tot>-1e30) &
             ep_tot0 = ep_tot0 + srclist(i)%m2_tot
          if (srclist(i)%m2_tot_err>-1e30) &
             ep_tot_err0 = ep_tot_err0 + srclist(i)%m2_tot_err**2 
          if (ep_tot0>0.) then
             srclist(i)%ep_tot = ep_tot0
             srclist(i)%ep_tot_err = sqrt(ep_tot_err0)
          end if
       end if       
       write(line(ip:),"(2(f9.5,1x))") srclist(i)%ep_tot,srclist(i)%ep_tot_err
       ip=ip+20 

      if (sing_inst) then
         ep_xid0=0.
         ep_xid_err0=0.
         if (srclist(i)%pn_xid>-1e30) &
            ep_xid0 = ep_xid0 + srclist(i)%pn_xid
         if (srclist(i)%pn_xid_err>-1e30) &
            ep_xid_err0 = ep_xid_err0 + srclist(i)%pn_xid_err**2
         if (srclist(i)%m1_xid>-1e30) &
            ep_xid0 = ep_xid0 + srclist(i)%m1_xid
         if (srclist(i)%m1_xid_err>-1e30) &
            ep_xid_err0 = ep_xid_err0 + srclist(i)%m1_xid_err**2
         if (srclist(i)%m2_xid>-1e30) &
            ep_xid0 = ep_xid0 + srclist(i)%m2_xid
         if (srclist(i)%m2_xid_err>-1e30) &
            ep_xid_err0 = ep_xid_err0 + srclist(i)%m2_xid_err**2
         if (ep_xid0>0.) then
            srclist(i)%ep_xid = ep_xid0
            srclist(i)%ep_xid_err = sqrt(ep_xid_err0)
         end if
       end if
       if (srclist(i)%ep_xid_err>-1e30) then 
         write(line(ip:),"(2(f9.5,1x))") srclist(i)%ep_xid,srclist(i)%ep_xid_err
         ip=ip+20
       else
         write(line(ip:),"(2a)")"      -   ","      -   "
         ip=ip+20
       end if 

       if (srclist(i)%ep_extent>0.) then
         write(line(ip:),"(f8.1,2x)")srclist(i)%ep_extent
         ip=ip+10
       else
         write(line(ip:),"(a)")"      -   "
         ip=ip+10
       end if
       if (srclist(i)%ep_ext_err>0.) then
         write(line(ip:),"(f8.1,2x)") srclist(i)%ep_ext_err
         ip=ip+10
       else
         write(line(ip:),"(a)")"      -   "
         ip=ip+10
       end if
       if (sing_inst) then
          ep_ext_ml0 = 0.
          do j=1,nsets
             if (srclist(i)%pn_ext_ml(j)>-1e30) &
               ep_ext_ml0 = ep_ext_ml0 + srclist(i)%pn_ext_ml(j)
             if (srclist(i)%m1_ext_ml(j)>-1e30) &
               ep_ext_ml0 = ep_ext_ml0 + srclist(i)%m1_ext_ml(j)
             if (srclist(i)%m2_ext_ml(j)>-1e30) &
               ep_ext_ml0 = ep_ext_ml0 + srclist(i)%m2_ext_ml(j)
          end do
          if (ep_ext_ml0>0.) srclist(i)%ep_ext_ml(1)=ep_ext_ml0
          if (srclist(i)%ep_ext_ml(1)>0.) then 
            write(line(ip:),"(f8.1,2x)")srclist(i)%ep_ext_ml(1)
            ip=ip+10
          else
            write(line(ip:),"(a)")"      -   "
            ip=ip+10
          end if
       else      
          do j=1,header%nsets
             write(line(ip:),"(f8.1,2x)")srclist(i)%ep_ext_ml(j)
             ip=ip+10
          end do
       end if
       write(line(ip:),"(a)")srclist(i)%omflag
       ip=ip+10
        write(7,"(a)")line(1:ip) 
    end do
    write (7,"(a)") "</PRE>"
    write (7,"(a)") "</FONT></TABLE>"
    write (7,"(a)") "</CENTER></BODY></HTML>"

    close (7)
    
  end subroutine write_html_list


  !--------------------------------------------------------------------
  subroutine write_html_xid(filename,srclist, nsrcs, header, nsets)
  !--------------------------------------------------------------------  


    character(*), intent(in) :: filename
    type(Source_type), dimension(:),intent(inout) :: srclist
    integer(kind=int32) , intent (in) :: nsrcs, nsets
    type(Header_type) , intent(in) :: header
    integer(kind=int32) :: i,j,ip,left

    character(10) :: forma
    character(1000) :: line
    character(25) :: coostring

    real(kind=single) :: ep_tot0, ep_tot_err0, &
                         ep_xid0, ep_xid_err0, ep_ext_ml0
    integer(kind=int32), dimension(0:3) :: ep
    logical :: sing_inst

    open(unit=7,file=filename,form="formatted",status="unknown")
    
    if (header%ncrossid == 0 .and. nsets <= 3) then
      sing_inst=.TRUE.
      ep=0
      do j=1,nsets
        ep(header%id_inst(j))=ep(header%id_inst(j))+1
      end do
      do i=1,3
        if (ep(i)>1) sing_inst=.FALSE.
      end do
    else
      sing_inst=.FALSE.
    end if

    write (7,"(a)") "<HTML><HEAD>"
    write (7,"(a)") &
     "<TITLE>EPIC HTML summary source list</TITLE>"
    write (7,"(a)") "</head>"
    write (7,"(a)") "<BODY><CENTER>"
    write (7,"(a)") "<H2>Final EPIC observation source list</H2>"

    write (7,"(a)") "<PRE>"

    ! write column headings:

    do ip=1,200
      line(ip:ip)=" "
    end do 
    ip=1 

    write (line(ip:),"(a)") "  SRC_NUM "
    ip=ip+10
    write (line(ip:),"(a)") " EP_ML_ID " 
    ip=ip+10
    write (line(ip:),"(a)") " EP_BOX_ID " 
    ip=ip+10
!    write(line(ip:),"(3a)") "    RA    ","    DEC   "," RADEC_ERR"
!    ip=ip+30 
    write(line(ip:),"(3a)") "    RA       ","    DEC     "," RADEC_ERR"
    ip=ip+35 
    write(line(ip:),"(2a)") "  EP_TOT  ","EP_TOT_ERR"
    ip=ip+20 
    write(line(ip:),"(2a)") "  EP_XID  ","EP_XID_ERR"
    ip=ip+20 
    write(line(ip:),"(2a)") " EP_DET_ML"," EP_XID_ML"
    ip=ip+20 
    write(line(ip:),"(3a)") "  EP_EXT  ","EP_EXT_ERR"," EP_EXT_ML"
    ip=ip+30
!    write(line(ip:),"(3a)") "  PN_FLAG   ","  M1_FLAG   ","  M2_FLAG   "
!    ip=ip+36
    write(line(ip:),"(a)") "  EP_FLAG   "
    ip=ip+13
    write (line(ip:),"(a)") "  OMFLAG  "
    ip=ip+10

     write (7,"(a,/)")line(1:ip)

 ! write units

      do ip=1,200
        line(ip:ip)=" "
      end do 
      ip=1 

    write (line(ip:),"(a)") " "
    ip=ip+10
    write (line(ip:),"(a)") " " 
    ip=ip+10
    write (line(ip:),"(a)") " " 
    ip=ip+10
    write(line(ip:),"(3a)") "   hrm       ","    dms     ","  arcsec  "
    ip=ip+35 
    write(line(ip:),"(2a)") " cts/sec  "," cts/sec  "
    ip=ip+20 
    write(line(ip:),"(2a)") " cts/sec  "," cts/sec  "
    ip=ip+20 
    write(line(ip:),"(2a)") " "," "
    ip=ip+20 
    write(line(ip:),"(3a)") "image pix ","image pix "," "
    ip=ip+30
!    write(line(ip:),"(3a)") "            ","            ","            "
!    ip=ip+36
    write(line(ip:),"(a)") "            "
    ip=ip+13
    write (line(ip:),"(a)") "          "
    ip=ip+10

     write (7,"(a,/)")line(1:ip)
 ! write sources row by row

    do i=1,nsrcs
      do ip=1,200
        line(ip:ip)=" "
      end do 
      ip=1 

       left = int(log10(real(i)))+1
       write (forma,"(a,i1,a)") "(a,i",left,",a)" 
       write (line(ip:),forma) '<A NAME="SRC_',i,'">'
       ip=ip+15+left

       write(line(ip:),"(i7,3x)")i
       ip=ip+10


       write(line(ip:),"(i7,3x)") srclist(i)%ep_ml_id_src(1)
       ip=ip+10
       write(line(ip:),"(i7,3x)") srclist(i)%ep_box_id_src(1)
       ip=ip+10

       coostring=dec_to_sex(srclist(i)%ra,srclist(i)%dec)

!       write (line(ip:),"(2f10.5)")srclist(i)%ra,srclist(i)%dec
!       ip=ip+20
       write (line(ip:),"(A25)") coostring
       ip=ip+25
       write (line(ip:),"(f8.2,2x)")srclist(i)%radec_err
       ip=ip+10
       if (sing_inst) then
          ep_tot0=0. 
          ep_tot_err0=0.
          if (srclist(i)%pn_tot>-1e30) &
             ep_tot0 = ep_tot0 + srclist(i)%pn_tot
          if (srclist(i)%pn_tot_err>-1e30) &
             ep_tot_err0 = ep_tot_err0 + srclist(i)%pn_tot_err**2 
          if (srclist(i)%m1_tot>-1e30) &
             ep_tot0 = ep_tot0 + srclist(i)%m1_tot
          if (srclist(i)%m1_tot_err>-1e30) &
             ep_tot_err0 = ep_tot_err0 + srclist(i)%m1_tot_err**2 
          if (srclist(i)%m2_tot>-1e30) &
             ep_tot0 = ep_tot0 + srclist(i)%m2_tot
          if (srclist(i)%m2_tot_err>-1e30) &
             ep_tot_err0 = ep_tot_err0 + srclist(i)%m2_tot_err**2 
          if (ep_tot0>0.) then
             srclist(i)%ep_tot = ep_tot0
             srclist(i)%ep_tot_err = sqrt(ep_tot_err0)
          end if
       end if       


       write(line(ip:),"(2(f9.5,1x))") srclist(i)%ep_tot,srclist(i)%ep_tot_err
       ip=ip+20

       if (srclist(i)%ep_xid_err>-1e30) then 
         write(line(ip:),"(2(f9.5,1x))") srclist(i)%ep_xid,srclist(i)%ep_xid_err
         ip=ip+20
       else
         write(line(ip:),"(2a)") "    -     ","    -     "
         ip=ip+20
       end if 

       if (srclist(i)%ep_det_ml(1)>-1e30) then 
         write(line(ip:),"(f9.1,1x)") srclist(i)%ep_det_ml(1)
         ip=ip+10
       else
         write(line(ip:),"(a)")"      -   "
         ip=ip+10
       endif




       if (srclist(i)%ep_xid_ml(2)>-1e30) then 
         write(line(ip:),"(f9.1,1x)") srclist(i)%ep_xid_ml(2)
         ip=ip+10
       else
         write(line(ip:),"(2a)")"      -   ","      -   "
         ip=ip+10
       endif

       if (srclist(i)%ep_extent>0.) then
         write(line(ip:),"(f8.1,2x)")srclist(i)%ep_extent
         ip=ip+10
       else
         write(line(ip:),"(a)")"      -   "
         ip=ip+10
       end if
       if (srclist(i)%ep_ext_err>0.) then
         write(line(ip:),"(f8.1,2x)") srclist(i)%ep_ext_err
         ip=ip+10
       else
         write(line(ip:),"(a)")"      -   "
         ip=ip+10
       end if
       if (srclist(i)%ep_ext_ml(1) > 0.) then
         write(line(ip:),"(f8.1,2x)")srclist(i)%ep_ext_ml(1)
         ip=ip+10
       else
         write(line(ip:),"(a)")"      -   "
         ip=ip+10
       end if

!         write(line(ip:),"(a)")srclist(i)%pn_val_flag
!         ip=ip+12
!         write(line(ip:),"(a)")srclist(i)%m1_val_flag
!         ip=ip+12
!         write(line(ip:),"(a)")srclist(i)%m2_val_flag
!         ip=ip+12
       write(line(ip:),"(a)")srclist(i)%ep_flag
       ip=ip+13
       write(line(ip:),"(a)")srclist(i)%omflag
       ip=ip+10
        write(7,"(a)")line(1:ip) 
    end do
    write (7,"(a)") "</PRE>"
    write (7,"(a)") "</FONT></TABLE>"
    write (7,"(a)") "</CENTER></BODY></HTML>"

    close (7)
    
  end subroutine write_html_xid

  !--------------------------------------------------------------------
  function dec_to_sex(radec,decdec) result(coords)
  !--------------------------------------------------------------------

  character(25) :: coords
  character(1)  :: csig
  character(2)  :: crh,crm,cdd,cdm
  character(5)  :: crs
  character(4)  :: cds


  real(kind=double) :: radec, decdec, decsec, rasec, ra24, rr, dd

  integer(kind=int32) :: rahr,ramin,decdeg,decmin
  
  ra24=radec/15.D0
  rahr=int(ra24)
  rr=mod(ra24,1.D0)*60.D0
  ramin=int(rr)
  rasec=mod(rr,1.D0)*60.D0

  csig='+'
  if (decdec < 0.D0)  csig='-'
  decdec=abs(decdec)
  decdeg=int(decdec)
  dd=mod(decdec,1.D0)*60.D0
  decmin=int(dd)
  decsec=mod(dd,1.D0)*60.D0

  write(crh,'(I2.2)') rahr
  write(crm,'(I2.2)') ramin
  write(crs,'(F5.2)') rasec
  if (rasec < 10.D0)   write(crs,'("0",F4.2)') rasec
  write(cdd,'(I2.2)') decdeg
  write(cdm,'(I2.2)') decmin
  write(cds,'(F4.1)') decsec
  if (decsec < 10.D0)  write(cds,'("0",F3.1)') decsec

  coords='  '//crh//':'//crm//':'//crs//' '//csig//cdd//':'//cdm//':'//cds
  

  end function  dec_to_sex
       
  !--------------------------------------------------------------------
  subroutine init_list(srclist, nsrcs)
  !--------------------------------------------------------------------  
    type(Source_type), dimension(:), intent(in out) :: srclist
    integer(kind=int32),intent(in) :: nsrcs
    integer(kind=int32) :: i,j

    do i=1,nsrcs
     srclist(i)%id_src = null_entry
     srclist(i)%id_inst = null_entry
     srclist(i)%id_band = null_entry
     srclist(i)%src_num = null_entry
     do j=1, max_nsets
        srclist(i)%ep_ml_id_src(j) = null_entry
        srclist(i)%pn_ml_id_src(j) = null_entry
        srclist(i)%m1_ml_id_src(j) = null_entry
        srclist(i)%m2_ml_id_src(j) = null_entry
        srclist(i)%ep_box_id_src(j) = null_entry 
        srclist(i)%pn_box_id_src(j) = null_entry 
        srclist(i)%m1_box_id_src(j) = null_entry
        srclist(i)%m2_box_id_src(j) = null_entry
        srclist(i)%pn_det_ml(j) = 0.0
        srclist(i)%m1_det_ml(j) = 0.0 
        srclist(i)%m2_det_ml(j) = 0.0
        srclist(i)%pn_xid_ml(j) = 0.0
        srclist(i)%m1_xid_ml(j) = 0.0 
        srclist(i)%m2_xid_ml(j) = 0.0 
        srclist(i)%ep_det_ml(j) = 0.0 
        srclist(i)%pn_ext_ml(j) = 0.0   
        srclist(i)%m1_ext_ml(j) = 0.0 
        srclist(i)%m2_ext_ml(j) = 0.0 
        srclist(i)%ep_ext_ml(j) = 0.0
     end do
     srclist(i)%ra=0.0
     srclist(i)%dec = 0.0
     srclist(i)%ra_corr = 0.0
     srclist(i)%dec_corr = 0.0
     srclist(i)%lii =0.0
     srclist(i)%bii =0.0
     srclist(i)%lii_corr=0.0
     srclist(i)%bii_corr=0.0
     srclist(i)%radec_err=0.0
     srclist(i)%x_ima=0.0
     srclist(i)%x_ima_err=0.0
     srclist(i)%y_ima=0.0
     srclist(i)%y_ima_err=0.0

     srclist(i)%pn_tot_flux = -9.999e30  
     srclist(i)%err_pn_tot_flux = -9.999e30
     srclist(i)%m1_tot_flux = -9.999e30
     srclist(i)%err_m1_tot_flux = -9.999e30
     srclist(i)%m2_tot_flux = -9.999e30
     srclist(i)%err_m2_tot_flux = -9.999e30

     srclist(i)%pn_scts = -9.999e30  
     srclist(i)%pn_scts_err = -9.999e30
     srclist(i)%m1_scts = -9.999e30
     srclist(i)%m1_scts_err = -9.999e30
     srclist(i)%m2_scts = -9.999e30
     srclist(i)%m2_scts_err = -9.999e30

     srclist(i)%pn_tot = -9.999e30  
     srclist(i)%pn_tot_err = -9.999e30
     srclist(i)%m1_tot = -9.999e30
     srclist(i)%m1_tot_err = -9.999e30
     srclist(i)%m2_tot = -9.999e30
     srclist(i)%m2_tot_err = -9.999e30
     srclist(i)%ep_tot = -9.999e30
     srclist(i)%ep_tot_err = -9.999e30

     srclist(i)%pn_1_flux = -9.999e30  
     srclist(i)%err_pn_1_flux = -9.999e30
     srclist(i)%m1_1_flux = -9.999e30
     srclist(i)%err_m1_1_flux = -9.999e30
     srclist(i)%m2_1_flux = -9.999e30
     srclist(i)%err_m2_1_flux = -9.999e30

     srclist(i)%pn_1 = -9.999e30  
     srclist(i)%pn_1_err = -9.999e30
     srclist(i)%m1_1 = -9.999e30
     srclist(i)%m1_1_err = -9.999e30
     srclist(i)%m2_1 = -9.999e30
     srclist(i)%m2_1_err = -9.999e30         

     srclist(i)%pn_2_flux = -9.999e30  
     srclist(i)%err_pn_2_flux = -9.999e30
     srclist(i)%m1_2_flux = -9.999e30
     srclist(i)%err_m1_2_flux = -9.999e30
     srclist(i)%m2_2_flux = -9.999e30
     srclist(i)%err_m2_2_flux = -9.999e30

     srclist(i)%pn_2 = -9.999e30  
     srclist(i)%pn_2_err = -9.999e30
     srclist(i)%m1_2 = -9.999e30
     srclist(i)%m1_2_err = -9.999e30
     srclist(i)%m2_2 = -9.999e30
     srclist(i)%m2_2_err = -9.999e30

     srclist(i)%pn_3_flux = -9.999e30  
     srclist(i)%err_pn_3_flux = -9.999e30
     srclist(i)%m1_3_flux = -9.999e30
     srclist(i)%err_m1_3_flux = -9.999e30
     srclist(i)%m2_3_flux = -9.999e30
     srclist(i)%err_m2_3_flux = -9.999e30

     srclist(i)%pn_3 = -9.999e30  
     srclist(i)%pn_3_err = -9.999e30
     srclist(i)%m1_3 = -9.999e30
     srclist(i)%m1_3_err = -9.999e30
     srclist(i)%m2_3 = -9.999e30
     srclist(i)%m2_3_err = -9.999e30

     srclist(i)%pn_4_flux = -9.999e30  
     srclist(i)%err_pn_4_flux = -9.999e30
     srclist(i)%m1_4_flux = -9.999e30
     srclist(i)%err_m1_4_flux = -9.999e30
     srclist(i)%m2_4_flux = -9.999e30
     srclist(i)%err_m2_4_flux = -9.999e30

     srclist(i)%pn_4 = -9.999e30  
     srclist(i)%pn_4_err = -9.999e30
     srclist(i)%m1_4 = -9.999e30
     srclist(i)%m1_4_err = -9.999e30
     srclist(i)%m2_4 = -9.999e30
     srclist(i)%m2_4_err = -9.999e30

     srclist(i)%pn_5_flux = -9.999e30  
     srclist(i)%err_pn_5_flux = -9.999e30
     srclist(i)%m1_5_flux = -9.999e30
     srclist(i)%err_m1_5_flux = -9.999e30
     srclist(i)%m2_5_flux = -9.999e30
     srclist(i)%err_m2_5_flux = -9.999e30

     srclist(i)%pn_5 = -9.999e30  
     srclist(i)%pn_5_err = -9.999e30
     srclist(i)%m1_5 = -9.999e30
     srclist(i)%m1_5_err = -9.999e30
     srclist(i)%m2_5 = -9.999e30
     srclist(i)%m2_5_err = -9.999e30

!     srclist(i)%pn_6 = -9.999e30  
!     srclist(i)%pn_6_err = -9.999e30
!     srclist(i)%m1_6 = -9.999e30
!     srclist(i)%m1_6_err = -9.999e30
!     srclist(i)%m2_6 = -9.999e30
!     srclist(i)%m2_6_err = -9.999e30


     srclist(i)%pn_xid_flux = -9.999e30  
     srclist(i)%err_pn_xid_flux = -9.999e30
     srclist(i)%m1_xid_flux = -9.999e30
     srclist(i)%err_m1_xid_flux = -9.999e30
     srclist(i)%m2_xid_flux = -9.999e30
     srclist(i)%err_m2_xid_flux = -9.999e30

     srclist(i)%ep_1_flux = -9.999e30  
     srclist(i)%err_ep_1_flux = -9.999e30
     srclist(i)%ep_2_flux = -9.999e30  
     srclist(i)%err_ep_2_flux = -9.999e30
     srclist(i)%ep_3_flux = -9.999e30  
     srclist(i)%err_ep_3_flux = -9.999e30
     srclist(i)%ep_4_flux = -9.999e30  
     srclist(i)%err_ep_4_flux = -9.999e30
     srclist(i)%ep_5_flux = -9.999e30  
     srclist(i)%err_ep_5_flux = -9.999e30
     srclist(i)%ep_tot_flux = -9.999e30  
     srclist(i)%err_ep_tot_flux = -9.999e30
     srclist(i)%ep_xid_flux = -9.999e30  
     srclist(i)%err_ep_xid_flux = -9.999e30

     srclist(i)%pn_xid = -9.999e30
     srclist(i)%pn_xid_err  = -9.999e30
     srclist(i)%m1_xid  = -9.999e30
     srclist(i)%m1_xid_err  = -9.999e30
     srclist(i)%m2_xid = -9.999e30
     srclist(i)%m2_xid_err = -9.999e30
     srclist(i)%ep_xid = -9.999e30
     srclist(i)%ep_xid_err = -9.999e30
     srclist(i)%pn_extent = -9.999e30 
     srclist(i)%pn_ext_err = -9.999e30  
     srclist(i)%m1_extent = -9.999e30 
     srclist(i)%m1_ext_err = -9.999e30
     srclist(i)%m2_extent = -9.999e30  
     srclist(i)%m2_ext_err = -9.999e30  
     srclist(i)%ep_extent = -9.999e30  
     srclist(i)%ep_ext_err = -9.999e30


     srclist(i)%pn_1_ml = -9.999e30 
     srclist(i)%m1_1_ml = -9.999e30
     srclist(i)%m2_1_ml = -9.999e30
     srclist(i)%pn_2_ml = -9.999e30   
     srclist(i)%m1_2_ml = -9.999e30
     srclist(i)%m2_2_ml = -9.999e30
     srclist(i)%pn_3_ml = -9.999e30   
     srclist(i)%m1_3_ml = -9.999e30
     srclist(i)%m2_3_ml = -9.999e30
     srclist(i)%pn_4_ml = -9.999e30   
     srclist(i)%m1_4_ml = -9.999e30
     srclist(i)%m2_4_ml = -9.999e30
     srclist(i)%pn_5_ml = -9.999e30   
     srclist(i)%m1_5_ml = -9.999e30
     srclist(i)%m2_5_ml = -9.999e30


     srclist(i)%pn_1_exp = -9.999e30 
     srclist(i)%m1_1_exp = -9.999e30
     srclist(i)%m2_1_exp = -9.999e30
     srclist(i)%pn_2_exp = -9.999e30   
     srclist(i)%m1_2_exp = -9.999e30
     srclist(i)%m2_2_exp = -9.999e30
     srclist(i)%pn_3_exp = -9.999e30   
     srclist(i)%m1_3_exp = -9.999e30
     srclist(i)%m2_3_exp = -9.999e30
     srclist(i)%pn_4_exp = -9.999e30   
     srclist(i)%m1_4_exp = -9.999e30
     srclist(i)%m2_4_exp = -9.999e30
     srclist(i)%pn_5_exp = -9.999e30   
     srclist(i)%m1_5_exp = -9.999e30
     srclist(i)%m2_5_exp = -9.999e30
!     srclist(i)%pn_6_exp = -9.999e30   
!     srclist(i)%m1_6_exp = -9.999e30
!     srclist(i)%m2_6_exp = -9.999e30

     srclist(i)%pn_1_vig = -9.999e30 
     srclist(i)%m1_1_vig = -9.999e30
     srclist(i)%m2_1_vig = -9.999e30
     srclist(i)%pn_2_vig = -9.999e30   
     srclist(i)%m1_2_vig = -9.999e30
     srclist(i)%m2_2_vig = -9.999e30
     srclist(i)%pn_3_vig = -9.999e30   
     srclist(i)%m1_3_vig = -9.999e30
     srclist(i)%m2_3_vig = -9.999e30
     srclist(i)%pn_4_vig = -9.999e30   
     srclist(i)%m1_4_vig = -9.999e30
     srclist(i)%m2_4_vig = -9.999e30
     srclist(i)%pn_5_vig = -9.999e30   
     srclist(i)%m1_5_vig = -9.999e30
     srclist(i)%m2_5_vig = -9.999e30

     srclist(i)%pn_ontime = -9.999e30   
     srclist(i)%m1_ontime = -9.999e30
     srclist(i)%m2_ontime = -9.999e30
     srclist(i)%pn_maskfrac = -9.999e30
     srclist(i)%m1_maskfrac = -9.999e30
     srclist(i)%m2_maskfrac= -9.999e30
     srclist(i)%pn_offax  = -9.999e30
     srclist(i)%m1_offax  = -9.999e30
     srclist(i)%m2_offax  = -9.999e30
     srclist(i)%pn_hr1  = -9.999e30
     srclist(i)%pn_hr1_err = -9.999e30
     srclist(i)%pn_hr2 = -9.999e30
     srclist(i)%pn_hr2_err = -9.999e30
     srclist(i)%pn_hr3 = -9.999e30
     srclist(i)%pn_hr3_err = -9.999e30
     srclist(i)%pn_hr4 = -9.999e30
     srclist(i)%pn_hr4_err = -9.999e30
     srclist(i)%m1_hr1 = -9.999e30
     srclist(i)%m1_hr1_err = -9.999e30  
     srclist(i)%m1_hr2 = -9.999e30
     srclist(i)%m1_hr2_err = -9.999e30
     srclist(i)%m1_hr3 = -9.999e30
     srclist(i)%m1_hr3_err = -9.999e30
     srclist(i)%m1_hr4 = -9.999e30
     srclist(i)%m1_hr4_err = -9.999e30
     srclist(i)%m2_hr1 = -9.999e30
     srclist(i)%m2_hr1_err = -9.999e30 
     srclist(i)%m2_hr2 = -9.999e30
     srclist(i)%m2_hr2_err= -9.999e30
     srclist(i)%m2_hr3  = -9.999e30
     srclist(i)%m2_hr3_err= -9.999e30
     srclist(i)%m2_hr4  = -9.999e30
     srclist(i)%m2_hr4_err= -9.999e30
     srclist(i)%ep_hr1  = -9.999e30
     srclist(i)%ep_hr1_err  = -9.999e30
     srclist(i)%ep_hr2  = -9.999e30
     srclist(i)%ep_hr2_err= -9.999e30
     srclist(i)%ep_hr3  = -9.999e30
     srclist(i)%ep_hr3_err= -9.999e30
     srclist(i)%ep_hr4  = -9.999e30
     srclist(i)%ep_hr4_err= -9.999e30

     srclist(i)%pn_pileup = -9.999e30
     srclist(i)%m1_pileup = -9.999e30
     srclist(i)%m2_pileup = -9.999e30

     srclist(i)%var_stat = 0.0
     srclist(i)%p_var = 0.0
     srclist(i)%n_bins = 0 
     srclist(i)%var_exp_no = null_entry
     do j=1, 5
        srclist(i)%om_id(j) = null_entry
        srclist(i)%om_id_dist(j) = -9.999e30
     end do
     srclist(i)%n_opt_co = 0
     srclist(i)%omflag = 'NO_OM_DATA'
     srclist(i)%spectra = .FALSE.  
     srclist(i)%tseries = .FALSE. 
      srclist(i)%pn_val_flag = '-' 
      srclist(i)%pn_ver_flag = '-' 
      srclist(i)%pn_ver_comm = '-' 
      srclist(i)%m1_val_flag = '-'  
      srclist(i)%m1_ver_flag = '-' 
      srclist(i)%m1_ver_comm = '-'
      srclist(i)%m2_val_flag = '-'  
      srclist(i)%m2_ver_flag = '-' 
      srclist(i)%m2_ver_comm = '-' 
      srclist(i)%ep_flag = '-' 
    end do

  end subroutine init_list

end module srcmatch_io


